/********************************************************************
Chebyshev filter
********************************************************************/

#include <stdio.h>
#include <math.h>

#define	CANCEL 11
#define	NO     22

#define CAUSAL 1
#define UNCAUSAL 2


/*----------------------------------------------------------------*/
void recfil(int n, int k, int filt_m, 
	    float f_buf[], double h[], int *canc, int nml)
{
    /*c
      c	  recursive filtering : f(z) = (1+a*z+aa*z**2)/(1+b*z+bb*z**2)
      c
      c	  arguments
      c	    x   :input time series
      c	    y   :output time series (may be equivalent to x)
      c	    n   :length of x & y
      c	    h   :filifr coeficients :h(1)=a,h(2)=aa,h(3)=b,h(4)=bb
      c	    nml :>0 ; for normal  direction filtering
      c	         <0 ; for reverse direction filtering
      c
      c	  m,saito  (6/XII/75)
      c
    */

    /***** Variables *****/
    int i, j, jd, l;
    double  a, aa, b, bb;
    double  u1, u2, u3;
    double  v1, v2, v3;
  
    /********** Main **********/
    l = (k - 1) * 4;
    if (nml < 0) {
	/*****	   reverse filtering   */
	j  = n - 1;
	jd =-1;
    } else {
	/*****	   normal filtering    */
	j = 0;
	jd= 1;
    }
  
    a =h[l + 0];
    aa=h[l + 1];
    b =h[l + 2];
    bb=h[l + 3];
    u1=0.0;
    u2=0.0;
    v1=0.0;
    v2=0.0;
    /*****	 filtering     */
    for(i = 1 ; i <= n ; i++){
	u3=u2;
	u2=u1;
	u1=f_buf[j];
	v3=v2;
	v2=v1;
	v1=u1+a*u2+aa*u3-b*v2-bb*v3;
	f_buf[j]=v1;
	j =j+jd;
    }
}
/*----------------------------------------------------*/
void tchlop(int *filt_m, double fp, double fs, double *filt_gn, double h[],
	    int *canc, double ap, double as)
{
    /*c
      c	Chebyshev Low-Pass Filter Coefficient (EQUI-RIPPLE STOP BAND)
      c
      c	Arguments
      c	 H	: Filter Coefficients
      c	 M	: Order Of Filter
      c	 GN	: Gain Factor
      c	 N	: Order Of Chebysev Polynomial
      c	 EPS	: Ripple
      c	 FP	: Pass Band Frequency (Non-Dimensional)
      c	 FS	: Stop Band Frequency
      c	 AP 	: Max, Attenuation In Pass Band
      c	 AS 	: Min, Attenuation In Pass Band
      c
      c	M. Saito (1/XI/77)
      c
    */
    /***** Variables *****/
    int j, k, ll, n;
    double	eps;
    double pi = M_PI;
    double hp = M_PI_2;
    double wp, ws, c, op, tp, pa, sa, ps, fn, g, ch, sh, dp, fj;
    double rj, sj, tj, st, cr, a, b;

    /***** Main *****/
    if(fabs(fp) > fabs(fs)) {
	wp = fabs(fs) * pi;
	ws = fabs(fp) * pi;
    } else {
	wp = fabs(fp) * pi;
	ws = fabs(fs) * pi;
    }
  
    if((wp == ws) || (wp == 0) || (ws >= hp)){
	/*c	   ERROR  */
	*canc = CANCEL;
	return;
    }
    c  = 1.0 / tan(ws);
    op = (1.0 / tan(wp)) / c;
    tp = log(op + sqrt((op + 1.0) * sin(ws - wp) / (cos(ws) * sin(wp))));

    if(fabs(ap) > fabs(as)) {
	pa = fabs(as);
	sa = fabs(ap);
    } else {
	pa = fabs(ap);
	sa = fabs(as);
    }
  
    if (pa == 0.0)  pa = 0.5;
    if (sa == 0.0)  sa = 10.0;
    ps = pa / sa;

    ll = fabs(log((1.0 + sqrt((1.0 - ps)*(1.0 + ps))) / ps) / tp) + 0.5;
    if(ll < 2)  
	n = 2;
    else	   
	n = ll;
    fn = (double)n;
    eps = sqrt(1.0 / (pa * sa * cosh((double)tp * (double)fn)));

    *filt_m  = n / 2;
    k  = *filt_m * 4;
    g  = 1.0;
    ps = log((1.0 + sqrt(1.0 + (eps*eps))) / eps) / fn;
    ch = cosh(ps);
    sh = sinh(ps);
    dp = hp / fn;
    fj = 1.0;

    for(j = 1 ; j <= *filt_m ; j ++){
	rj = cos(dp * fj);
	sj = rj * ch;
	tj = sin(dp * fj) * sh;
	st = (sj*sj) + (tj*tj);
	sj = (sj/st) * (sj/st);
	tj = tj / st;
	cr = c * rj;
	fj = fj + 2.0;
	a  = 1.0 / (((c + tj)*(c + tj)) + sj);
	b  = 1.0 + (cr*cr);
	g  = g * a * b / st;
	h[((j - 1) * 4)] = -2.0 * (cr - 1.0) * (cr + 1.0) / b;
	h[((j - 1) * 4 + 1)] = 1.0;
	h[((j - 1) * 4 + 2)] = -2.0 * ((c + tj) * (c - tj) - sj) * a;
	h[((j - 1) * 4 + 3)] = (((c - tj) * (c - tj)) + sj) * a;
    }
    *filt_gn = g;
    if((n % 2) == 0.0) return;
    /*c	  ODD N   */
    *filt_m += 1;
    *filt_gn /= 1.0 + sh*c;
    h[ k ] = 1.0;
    h[k+1] = 0.0;
    h[k+2] = (1.0 - sh * c) / (1.0 + sh * c); 
    h[k+3] = 0.0;
}


/*----------------------------------------------------*/
void tchhip(int *filt_m,double fp,double fs,double *filt_gn,double h[],
	    int *canc,double ap,double as)
{
    /*c
      c	  CHEBYSHEV  HIGH-PASS FILTER COEFFICIENT  (EQUI-RIPPLE STOP BAND)
      c
      c	  Arguments
      c	    h	: Filter Coefficients
      c	    m	: Order Of Filter
      c	    gn	: Gain Factor
      c  	    n	: Order Of Chebyshev Polynomial
      c	    eps : Ripple
      c	    fp	: Pass Band Frequency  (Non-Dimensional)
      c	    fs	: Stop Band Frequency
      c	    ap	: Max, Attenuation In Pass Band
      c	    as	: Min, Attenuation In Stop Band
      c
      c	M. Saito  (1/XI/77)
      c											*/
    int	j, k, ll, n;
    double	eps;
    double	pi = 3.141593;
    double	hp = 1.570796;
    double	wp, ws, c, op, tp, pa, sa, ps, fn, g, ch, sh, dp, fj;
    double	rj, sj, tj, st, cr, a, b;
    if(fabs(fp) < fabs(fs))
    {
	wp = fabs(fs) * pi;
	ws = fabs(fp) * pi;
    }
    else
    {
	wp = fabs(fp) * pi;
	ws = fabs(fs) * pi;
    }
  
    if ((wp == ws) || (ws == 0.0) || (wp >= hp)) goto error;
    /*c	  DETERMINE N,EPS & C	*/
    c  = tan(ws);
    op = tan(wp) / c;
    tp = log(op + sqrt((op + 1.0) * sin(wp - ws) / (cos(wp) * sin(ws))));
  
    if(fabs(ap) > fabs(as))
    {
	pa = fabs(as);
	sa = fabs(ap);
    }
    else
    {
	pa = fabs(ap);
	sa = fabs(as);
    }
  
    if(pa == 0.0)  pa = 0.5;
    if(sa == 0.0)  sa = 10.0;
    ps = pa / sa;
    ll = fabs(log((1.0 + sqrt((1.0 - ps)*(1.0 + ps))) / ps) / tp) + 0.5;
    if(ll < 2)  n = 2;
    else	   n = ll;
  
    fn = (double)n;
    eps = sqrt(1.0 / (pa * sa * cosh((double)tp * (double)fn)));
    /*c*/
    *filt_m  = n / 2;
    k  = *filt_m * 4;
    g  = 1.0;
    ps = log((1.0 + sqrt(1.0 + eps*eps)) / eps) / fn;
    ch = cosh(ps);
    sh = sinh(ps);
    dp = hp / fn;
    fj = 1.0;
    /*c*/
    for(j = 1 ; j <= *filt_m ; j ++)
    {
	rj = cos(dp * fj);
	sj = rj * ch;
	tj = sin(dp * fj) * sh;
	st = sj*sj + tj*tj;
	sj = (sj / st) * (sj / st);
	tj = tj / st;
	cr = c * rj;
	fj = fj + 2.0;
	a  = 1.0 / ((c + tj)*(c + tj) + sj);
	b  = 1.0 + cr*cr;
	g  = g * a * b / st;
	h[ ((j - 1) * 4) ] = 2.0 * (cr - 1.0) * (cr + 1.0) / b;
	h[((j - 1) * 4 + 1)] = 1.0;
	h[((j - 1) * 4 + 2)] = 2.0 * ((c + tj) * (c - tj) - sj) * a;
	h[((j - 1) * 4 + 3)] = ((c - tj) * (c - tj) + sj) * a;
    }
  
    /*c*/
    *filt_gn = g;
    if((n % 2) == 0)  return;
    /*c	 ODD N	*/
    *filt_m  += 1;
    *filt_gn /= 1.0 + sh*c;
    h[ k ] = -1.0;
    h[k+1] = 0.0;
    h[k+2] = -(1.0 - sh * c) / (1.0 + sh * c);
    h[k+3] = 0.0;
    return;
    /*c	  ERROR  */
 error:
    {
	fprintf(stderr,
		"ERROR occured. Invalid INPUT into \"tchhip\".  FP %f : FS = %f\n", fp,fs);
	*canc = CANCEL;
	return;
    }
}

/*----------------------------------------------------*/
void tchpas(int *filt_m,double fl,double fh,double fs,double *filt_gn,
	    double h[],int *canc,double ap,double as)
{
    double r[2];
    struct compl {
	double  cx;
	double  cy;
    }  q[2], oj, cq, tempo1;
    /*c
      c	CHEBYSHEV BAND-PASS FILTER COEFFICIENT  (EQUI-RIPPLE STOP BAND)
      c
      c	Arguments
      c	    h	: Filter Coefficients
      c	    m	: Order Of Filter
      c	    gn	: Gain Factor
      c	    n	: Order Of Chebyshev Polynomial
      c	    eps	: Ripple
      c	    fl	: Low Frequency Cut-Off  (Non-Dimensional)
      c	    fh	: High Frequency Cut-Off
      c	    fs  : Stop Band Frequency
      c	    ap	: Max, Attenuation In Pass Band
      c	    as	: Min, Attenuation In Stop Band
      c
      c	M. Saito  (1/XI/77)
      c										*/
    int	i, j, k, l, ll,n;
    double	eps;
    double	wl, wh, ws, tl, th, ts, c, cc, ww, clh, op, pa, sa, tp, ps, fn;
    double	fj, g, ch, sh, dp, rj, sj, tj, a, wpc, wmc, st, tempo_abs;
  
    if(fabs(fl) > fabs(fh))
    {
	wl = fabs(fh) * M_PI;
	wh = fabs(fl) * M_PI;
    }
    else
    {
	wl = fabs(fl) * M_PI;
	wh = fabs(fh) * M_PI;
    }
    ws = fabs(fs) * M_PI;
    /*printf("WL = %f : WH = %f : WS = %f\n", wl, wh, ws);*/
    if((wl == wh) || (wl == 0.0) || (wh >= M_PI_2) || (ws == 0.0)
       || (ws >= M_PI_2) || ((ws - wl) * (ws - wh) <= 0.0))  goto error;
    /*c	DETERMINE  N,EPS & C   */
    tl = tan(wl);
    th = tan(wh);
    ts = tan(ws);
    c  = 1.0 / fabs(ts - tl * th / ts);
    cc = c*c;
    ww = cc * th * tl;
    clh= cos(wl) * cos(wh);
    op = clh / (c * sin(wh - wl));
    if(fabs(ap) > fabs(as))
    {
	pa = fabs(as);
	sa = fabs(ap);
    }
    else
    {
	pa = fabs(ap);
	sa = fabs(as);
    }
  
    if(pa == 0.0 )  pa = 0.5;
    if(sa == 0.0)  sa = 10.0;
    /*printf("PA = %f : SA = %f\n", pa, sa);*/
    tp = log(op + sqrt((op - 1.0) * (op + 1.0)));
    ps = pa / sa;
    ll = fabs(log((1.0 + sqrt((1.0 - ps) * (1.0 + ps))) / ps) / tp) + 0.5;
    if(ll < 2)  n = 2;
    else	   n = ll;
    /*printf("LL\(double\) = %f : LL = %d : N = %d\n", (fabs(log((1.0 + sqrt((1.0 - ps)*(1.0 + ps))) / ps) / tp) + 0.5), ll, n);*/
    fn = (double)n;
    eps= sqrt(1.0 / (pa * sa * cosh(tp * fn)));
    /*c*/
    k  = n / 2;
    *filt_m  = k * 2;
    /*printf("filter order = %d \n",*filt_m);
      printf(" k = %d\n",k);*/
    l  = 0;
    fj = 1.0;
    g  = 1.0;
    ps = log((1.0 + sqrt(1.0 + eps*eps)) / eps) / fn;
    ch = cosh(ps);
    sh = sinh(ps);
    dp = M_PI_2/fn;
    /*c*/
    for(j = 0 ; j < k ; j++)
    {
	/*	printf("loop J = %d\n",j);*/
	rj = cos(dp * fj);
	sj = rj * ch;
	tj = sin(dp * fj) * sh;
	st = sj*sj + tj*tj;
	rj = rj*rj;
	r[0] = ((1.0 + sqrt(1.0 +4.0 * ww * rj)) * 0.5)
	    * ((1.0 + sqrt(1.0 +4.0 * ww * rj)) * 0.5) / rj;
	r[1] = ww*ww / r[0];
	oj.cx = sj / st *0.5;
	oj.cy = tj / st *0.5;
	fj = fj + 2.0;
	tempo1.cx = ww + (oj.cx)*(oj.cx) - (oj.cy)*(oj.cy);
	tempo1.cy = 2 * oj.cx * oj.cy;
	tempo_abs = sqrt((tempo1.cx * tempo1.cx) + (tempo1.cy * tempo1.cy));
	cq.cx = sqrt(tempo_abs) * cos (0.5 * atan(tempo1.cy / tempo1.cx));
	cq.cy = sqrt(tempo_abs) * sin (0.5 * atan(tempo1.cy / tempo1.cx));
	q[0].cx = oj.cx + cq.cx;
	q[0].cy = oj.cy + cq.cy;
	q[1].cx = oj.cx - cq.cx;
	q[1].cy = oj.cy - cq.cy;
	g  = g * rj / st;
	/*c*/
	for(i = 0 ; i < 2 ; i++)
	{
	    /*		printf("loop I = %d\n",i);*/
	    sj = q[i].cx * q[i].cx;
	    tj = q[i].cy;
	    a  = 1.0 / ((c + tj)*(c + tj) + sj);
	    g  = g * (cc + r[i]) * a;
	    h[l] = -2.0 * (cc - r[i]) / (cc + r[i]);
	    h[l+1] = 1.0;
	    h[l+2] = 2. * ((tj + c) * (tj - c) + sj) * a;
	    h[l+3] = ((c - tj)*(c - tj) + sj) * a;
	    /*printf("H[%d]=%f:H[%d]=%f:H[%d]=%f:H[%d]=%f\n",l,h[l],l+1,h[l+1],l+2,h[l+2],l+3,h[l+3]);*/
	    l = l + 4;
	}
	/*c*/
    }
    /*c*/
    *filt_gn = g;
    if((n % 2) == 0)  return;
    /*c	  ODD N  */
    *filt_m += 1;
    wpc = cc * cos(wh - wl) / clh;
    wmc = -cc * cos(wh + wl) / clh;
    a  = 1.0 / (wpc + c / sh);
    *filt_gn *= c * a / sh;
    h[ l ] = 0.0;
    h[l+1] = -1.0;
    h[l+2] = 2.0 * wmc * a;
    h[l+3] = (wpc - c / sh) * a;
    /*printf("H[%d]=%f:H[%d]=%f:H[%d]=%f:H[%d]=%f\n",l,h[l],l+1,h[l+1],l+2,h[l+2],l+3,h[l+3]);*/
    return;
  
 error:
    /*c	  ERROR	*/
    fprintf(stderr,
	    " ERROR:  Invalid INPUT \"tchpas\".  FL : %f  FH : %f  FS : %f\n", fl, fh, fs);
    *canc = CANCEL;
    return;
}



/*----------------------------------------------------*/

void tchstp(int *filt_m,double fl,double fh,double fs,double *filt_gn,
	    double h[],int *canc,double ap,double as)
{
    double r[2];
    struct compl {
	double  cx;
	double  cy;
    }  q[2], oj, cq, tempo1;
    /*c
      c	CHEBYSHEV  BAND-STOP FILTER COEFFICIENT  (EQUI-RIPPLE STOP BAND)
      c
      c	Arguments
      c	  h	: Filter Coefficients
      c	  m	: Order Of Filter
      c	  gn	: Gain Factor
      c	  n	: Order Of Chebyshev Polynomial
      c	  eps	: Ripple
      c	  fl	: Low  Frequency Cut-Off  (Non-Dimensional)
      c	  fh	: High Frequency Cut-Off
      c	  fs	: Stop Band Frequency
      c	  ap	: Max, Attenuation In Pass Band
      c	  as	: Min, Attenuation In Stop Band
      c
      c	M. Saito  (1/XI/77)
      c											*/
    int 	i, j, k, l, ll,n;
    double	eps;
    double	hp = 1.570796;
    double	wl, wh, ws, tl, th, ts, c, cc, ww, clh, op, pa, sa, tp, ps, fn;
    double	fj, g, ch, sh, dp, rj, sj, tj, a, wpc, wmc, tempo_abs;
  
    if(fabs(fl) > fabs(fh))
    {
	wl = fabs(fh) * M_PI;
	wh = fabs(fl) * M_PI;
    }
    else
    {
	wl = fabs(fl) * M_PI;
	wh = fabs(fh) * M_PI;
    }
    ws = fabs(fs) * M_PI;
  
    if((wl == wh) || (wl == 0.0) || (wh >= hp) || ((ws - wl) * (ws - wh) >= 0.0))  goto error;
    /*c	  DETERMINE N,EPS & C  */
  
    tl = tan(wl);
    th = tan(wh);
    ts = tan(ws);
    c  = 1.0/fabs(ts - tl * th / ts);
    cc = c * c;
    ww = cc * tl * th;
    clh= 1.0 / (cos(wl) * cos(wh));
    op = c * sin(wh - wl) * clh;
    if(fabs(ap) > fabs(as))
    {
	pa = fabs(as);
	sa = fabs(ap);
    }
    else
    {
	pa = fabs(ap);
	sa = fabs(as);
    }
  
    if(pa == 0.0 )  pa = 0.5;
    if(sa == 0.0)  sa = 10.0;
    tp = log(op + sqrt((op - 1.0) * (op + 1.0)));
    ps = pa / sa;
    ll = fabs(log((1.0 + sqrt((1.0 - ps) * (1.0 + ps))) / ps) / tp) + 0.5;
    if(ll < 2)  n = 2;
    else	    n = ll;
    fn = (double)n;
    eps= sqrt(1.0 / (pa * sa * cosh(tp * fn)));
    /*c*/
    k  = n / 2;
    *filt_m  = k * 2;
    l  = 0;
    fj = 1.0;
    g  = 1.0;
    ps = log((1.0 + sqrt(1.0 + eps*eps)) / eps) / fn;
    ch = cosh(ps);
    sh = sinh(ps);
    dp = hp / fn;
    /*c*/
    for(j = 0 ; j <  k ; j++)
    {
	rj = cos(dp * fj);
	oj.cx = rj * ch * 0.5;
	oj.cy = - sin(dp * fj) * sh * 0.5;
	fj = fj+2.0;
	r[0] = (0.5 * (rj + sqrt(rj*rj + 4.0*ww))) * (0.5 * (rj + sqrt(rj*rj + 4.0*ww)));
	r[1] = ww*ww / r[0];
	tempo1.cx = ww + (oj.cx)*(oj.cx) - (oj.cy)*(oj.cy);
	tempo1.cy = 2 * oj.cx * oj.cy;
	tempo_abs = sqrt((tempo1.cx * tempo1.cx) + (tempo1.cy * tempo1.cy));
	cq.cx = sqrt(tempo_abs) * cos (0.5 * atan(tempo1.cy / tempo1.cx));
	cq.cy = sqrt(tempo_abs) * sin (0.5 * atan(tempo1.cy / tempo1.cx));
	q[0].cx = - oj.cx + cq.cx;
	q[0].cy = - oj.cy + cq.cy;
	q[1].cx = - oj.cx - cq.cx;
	q[1].cy = - oj.cy - cq.cy;
	/*c*/
	for(i = 0 ; i < 2 ; i++)
	{
	    sj = q[i].cx * q[i].cx;
	    tj = q[i].cy;
	    a  = 1.0 / ((c + tj)*(c + tj) + sj);
	    g  = g * (cc + r[i]) * a;
	    h[l] =- 2.0 * (cc - r[i]) / (cc + r[i]);
	    h[l+1] = 1.0;
	    h[l+2] = 2.0 * ((tj - c) * (tj + c) + sj) * a;
	    h[l+3] = ((tj - c)*(tj - c) + sj)*a;
	    l  = l + 4;
	}
	/*c*/
    }
    /*c*/
    *filt_gn = g;
    if((n % 2) == 0)  return;
    /*c*/
    *filt_m += 1;
    wpc = cc * cos(wh - wl) * clh;
    wmc = - cc * cos(wh + wl) * clh;
    a  = 1.0 / (wpc + c * sh);
    *filt_gn *= wpc * a;
    h[l] = 2.0 * wmc / wpc;
    h[l+1] = 1.0;
    h[l+2] = 2.0 * wmc * a;
    h[l+3] = (wpc - c * sh) * a;
    return;
    /*c	  ERROR  */
 error:
    fprintf(stderr,
	    " ERROR:  Invalid INPUT \"tchstp\".  FL : %f  FH : %f  FS : %f\n", fl, fh, fs);
    *canc = CANCEL;
    return;
}

float window(int pnt,int itwin,int winlob)
{
    double	val, rad;
    double	a0, a1, a2;
  
    a0 = 0.42659071;	a1 = 0.49656062;	a2 = 0.07684867;
  
    if(pnt<winlob){
	rad  = (double)(pnt-winlob) / (double)(2*winlob) * 2. * M_PI;
	val  = a0;
	val += a1 * cos(rad);
	val += a2 * cos(2.*rad);
    }	
    else if(pnt<(itwin+winlob))	val = 1.;
    else if(pnt<(itwin + 2*winlob)){
	rad  = (double)(pnt-(itwin+winlob-1))/(double)(2*winlob)*2.*M_PI;
	val  = a0;
	val += a1 * cos(rad);
	val += a2 *cos(2.*rad);
    }
    else val = 0.;
  
    return((float)val);
}

/*----------------------------------------------------*/
void tandem(int n, int filt_m, float f_buf[], double h[], int *canc, int nml)
{
    /*c
      c	 recursive filtering in series
      c
      c	 arguments
      c	   x      : input time series
      c	   y      : output time series  (may be equivalent to x)
      c	   n      : length of x & y
      c	   m      : order of filter
      c	   nml    : >0 ; for normal  direction filtering
      c	            <0 ;     reverse direction filtering
      c
      c	 subroutine required : recfil
      c
      c	 m.saito  (6/XII/7ARET 0 -5)
      c											*/

    /***** Variables *****/
    int	i = 1;

    /**********  **********/
    if (filt_m <= 0) {
	*canc = CANCEL;
	return;
    }
    /*c****	 1-st call       */
    recfil(n,i,filt_m,f_buf,h,canc,nml);
    if (filt_m <= 1) return;
  
    /*c****	 2-nd and after      */
    for(i = 2 ; i <= filt_m ; i++){
	recfil(n,i,filt_m,f_buf,h,canc,nml);
    }
}

void do_filter(int count, int filt_m, double filt_gn, double h[], int *canc, 
	       float a[], double sampling, int causal_type)
{
    /***** Local Variables *****/
    int i;
    int itwin, winlob;
    double powgn;

    /********** Main **********/
    /***** Windowing *****/
    winlob = floor(sampling*0.7);
    itwin = count - 2*winlob;
    for(i=0;i<count;i++) a[i] *= window(i,itwin,winlob);

    /***** apply filter *****/
    if(*canc!=CANCEL){
	tandem(count,filt_m,a,h,canc,1);
	if(*canc==CANCEL) return;
    }
    if(causal_type==UNCAUSAL){
	if(*canc!=CANCEL){
	    tandem(count,filt_m,a,h,canc,-1);
	    if(*canc==CANCEL)return;
	}
    }
    if(causal_type==UNCAUSAL) powgn = pow(filt_gn,2.);
    else powgn = filt_gn;
     
    for(i=0; i<count; i++) a[i] = a[i] * powgn;
}

