# --------------------------------------------------------------------
# Check parameters related to drawing.
# retunr 1 if invalid, otherwise return 0
# --------------------------------------------------------------------
proc check_parameter {} {
   # global --------------------
   global paste file_name wk gbl filt
   
   # main --------------------
   # mode check
   if { $paste(image,mode) == 2 } {
      # interpolate mode is valid only for VD, TwoTone
      if { $gbl(mode) < 10} {
	 put_error "(Warning) Can't use interpolate mode for this style.\nChange interpolate mode into image mode."
	 set paste(image,mode) 1
      }
   }
   
   # File exists?
   if {[file readable $file_name(wave)] == 0} {
      put_error "Cannot find wave-from data file. Aborted."
      return 1
   }
   if {[file readable $file_name(rh)] == 0 && $gbl(data,type) == 1} {
      put_error "Cannot find Record Header file. Aborted."
      return 1
   }
   # zooming in?
   if { [$wk(w,canvas,c) gettags zoom_mark] != "" } {
      # zoom !!
      if {[change_range_by_zoom]} {
	 return 1
      }
   } elseif { [string compare $wk(zoom,status) "out"] != 0 } {
      # clear zoom stack
      if {$paste(left) != $wk(zoom,left,$wk(zoom,count)) || \
	      $paste(right) != $wk(zoom,right,$wk(zoom,count)) || \
	      $paste(top) != $wk(zoom,top,$wk(zoom,count)) || \
	      $paste(bottom) != $wk(zoom,bottom,$wk(zoom,count)) } {
	 clear_zoom_stack
      }
   }
   
   # horizontal/vertical range check
   switch [check_coordinates] {
      1 {
	 put_error "Invalid display range(distance). Aborted."
	 return 1
      }
      2 {
	 put_error "Invalid display range(time). Aborted."
	 return 1
      }
      default {}
   }
   
   # Reduction Velocity
   if {$paste(reduction) < 0.0} {
      put_error "Invalid Reduction Velocity, Velocity will be set 0.0km/sec"
      set paste(reduction) 0.0
   }
   
   # update amp correction mode
   update_amp_correct

   # set stack parameters
   if { $gbl(check,stack) == 0 } { 
      # no stack
      set paste(stack) 0
   } else {
      if { $paste(stack,mode) == "hyperbola" } {
	 # hyperbola stack
	 set paste(stack) 1
      } else {
	 # slant stack
	 set paste(stack) 2
      }
   }

   # scale factor
   if {$paste(amp,amplify) == 0.0} {
      put_error "Invalid Amplitude factor. set to 1.0"
      set paste(amp,amplify) 1
   }
   
   # wave density
   set paste(wave_density)\
       [expr { $paste(wave_density) < 1 } ? 1 : $paste(wave_density)]

   # cut off amplitude
   if {[string length $paste(amp,cutoff)] < 1} {
      put_error "Invalid Cut Off amplitude. set to 0.0"
      set paste(amp,cutoff) 0
   }
   
   # filter parameters
   foreach f {f1 f2 f3 f4} {
      if { $filt($f) == "" } {
	 put_error "Invalid filter parameters, $f.  Aborted."
	 return 1
      }
   }
   # update title
   set paste(label,title) [make_title]
   return 0
}
# --------------------------------------------------------------------
# show error message in the message frame
# --------------------------------------------------------------------
proc put_error {message} {
   # main
   print_message [format "\nError : %s\n" $message]
}

# --------------------------------------------------------------------
# check horizontal/vertical range
# --------------------------------------------------------------------
proc check_coordinates {} {
   global paste
   
   if {$paste(left) > $paste(right)} {
      set tmp $paste(left)
      set paste(left) $paste(right)
      set paste(right) $tmp
   }
   if {$paste(left) >= $paste(right) || [string length $paste(left)] < 1} {
      return 1
   }
   if {$paste(top) > $paste(bottom)} {
      set tmp $paste(top)
      set paste(top) $paste(bottom)
      set paste(bottom) $tmp
   }
   if {$paste(top) >= $paste(bottom) || [string length $paste(top)] < 1} {
      return 2
   }
   return 0
}

# --------------------------------------------------------------------
# Check whether the drawing condition is the same as the previous
# --------------------------------------------------------------------
proc check_same_draw_condition {} {
   # global --------------------
   global wk file_name paste gbl filt
   
   # main --------------------

   # all re-read ----------

   # input file and x-axis or data type and byte
   if {$wk(prev,wave) != $file_name(wave) || $wk(prev,rh) != $file_name(rh)\
	   || $wk(prev,xaxis) != $gbl(data,xaxis)\
	   || $wk(prev,datatype) != $gbl(data,type)\
	   || $wk(prev,databyte) != $gbl(data,byte) } {
      # completely different
      return -20
   }
   # SU filter
   if {$wk(prev,su) != $gbl(check,sufilter)\
	   || $wk(prev,sumod) != $gbl(su,mod) } {
      # SU filter setting is different
      return -30
   }

   # SU filter with pick data
   if {$gbl(check,sufilter) } {
      foreach i $gbl(su,mod) {
	 set mod_name [lindex $i 0]
	 set state    [lindex $i 1]
	 if { $state == 1 } {
	    if { [lsearch {sutmixfilter supickstack sutracekill} $mod_name] >= 0 } {
	       return -30
	    }
	 }
      }
   }

   # several plot conditions
   if {$wk(prev,plotcond) != [list $paste(reduction)] ||\
	$wk(prev,correct_shot) != $paste(correct,shot_mode) } {
      return -40
   }

   # re-read waveform data, but not record header ----------

   # time range
   if { $wk(prev,tmin) > $paste(top) || $wk(prev,tmax) < $paste(bottom) } {
      # time range is out of cached data (re-read all the data)
      return -1
   }
   # offset range
   if {$gbl(su,autowindow) == 1 && 
       ( $wk(prev,xmin) != $paste(left) || $wk(prev,xmax) != $paste(right))} {
      # Re-read because of SU auto window
      return -2
   }
   # stacking
   if {$wk(prev,stack) != [list $gbl(check,stack)\
			       $paste(stack,mode) $paste(stack,range)\
			       $paste(stack,vel) $paste(stack,time)]} {
      # Stacking condition is different
      return -3
   }
   # time-domain filter
   if {$wk(prev,filter) !=\
	   [list $gbl(check,filter) $filt(type) $filt(causal) $filt(offset)\
		$filt(f1) $filt(f2) $filt(f3) $filt(f4) ] } {
      return -4
   }

   # no change ----------
   return 1
}

# --------------------------------------------------------------------
# Check whether the rh must be re-read.
# --------------------------------------------------------------------
proc check_same_rh_condition {} {
   return [expr { [check_same_draw_condition] < -10 } ? { 0 } : { 1 }]
}

# --------------------------------------------------------------------
# Preserve current drawing condition
# --------------------------------------------------------------------
proc preserve_draw_condition {} {
   # global --------------------
   global wk file_name paste gbl filt
   
   # main --------------------
   set wk(prev,datatype) $gbl(data,type)
   set wk(prev,databyte) $gbl(data,byte)
   set wk(prev,total_shot) $wk(total_shot)
   set wk(prev,wave) $file_name(wave) 
   set wk(prev,rh) $file_name(rh) 
   set wk(prev,su) $gbl(check,sufilter)
   set wk(prev,sumod) $gbl(su,mod)
   set wk(prev,xmin) $paste(left)
   set wk(prev,xmax) $paste(right)
   set wk(prev,plotcond) [list $paste(reduction)]
   set wk(prev,xaxis) $gbl(data,xaxis)
   set wk(prev,correct_shot) $paste(correct,shot_mode)
   set wk(prev,stack) [list $gbl(check,stack)\
			   $paste(stack,mode) $paste(stack,range)\
			   $paste(stack,vel) $paste(stack,time)]
   set wk(prev,filter) [list $gbl(check,filter)\
			    $filt(type) $filt(causal) $filt(offset)\
			    $filt(f1) $filt(f2) $filt(f3) $filt(f4) ]
}

proc clear_data_cache {} {
   global wk
   set wk(prev,wave) ""
   print_message "Cache data were cleared.\n"
}