#include <stdio.h>
#include <string.h>
#include <math.h>

#include "pasteup.h"

/********************************************************************
wiggle for GraphCanvas
--------------------------------------------------------------------
Inputs:
 n           : Dimension of a and t 
 a           : Amplitude array (waveform)
 t           : Time array
 x           : trace x-position (offset distance)
********************************************************************/
static void draw_graph_wiggle(Tcl_Interp *interp,
			      int n, float *a, float *t, double x)
{
    /********** Local Variables *********/
    register int i;
    double line_width;
    Tcl_Obj* coordlist;
    char com[128];

    /*************** Main **************/
    /** make coordinates list **/
    coordlist = Tcl_NewListObj(0, NULL);
    Tcl_SetVar2Ex(interp, "coordlist", NULL, coordlist, 0);
    for(i=0;i<n;i++){
	TCL_APPEND(coordlist, Double, x+ a[i]);
	TCL_APPEND(coordlist, Double, t[i]);
    }

    /** draw !! **/
    TCL_GET(Double, gbl, "line_width", line_width);
    sprintf(com,
	    "%s marker create line -coor $coordlist -linew %g -name wave%.3g",
	    TCL_GETSTRING(wk, "w,canvas,g"), line_width/5.0, x);
    Tcl_Eval(interp, com);
}

/********************************************************************
Variable Area with Red&Blue color for GraphCanvas
--------------------------------------------------------------------
Input:
 n           : dimension of a and t
 a           : Amplitude array (waveform)
 t           : Time array
 x           : trace x-position (offset distacne)
 threshold   : threshold level for VA
 col1        : plus side color
 col2        : minus side color
********************************************************************/
static void draw_graph_VA(Tcl_Interp *interp, int n, float *a, float *t,
			  double x, float threshold, int col1, int col2)
{
    /********** Local Variables *********/
    register int count1, i, j;
    float *va;
    float gain;
    char *g;
    Tcl_Obj* coordlist;
    char com[256];

    /******************** Main *******************/
    if(n<2) return; /* Too short trace */

    if( NULL == (va = (float *)malloc(n * sizeof(float))) ) {
	PASTEUP_ERROR;
    }
    
    g = TCL_GETSTRING(wk, "w,canvas,g");
    for(i=0;i<n;i++) a[i] += x;

    /***** declare tcl variable *****/
    coordlist = Tcl_NewListObj(0, NULL);
    Tcl_SetVar2Ex(interp, "coordlist", NULL, coordlist, 0);

    /***** plus side *****/
    /** threshold points **/
    gain = x + threshold;
    count1 = search_threshold(n, a, t, va, gain);
    /** plot **/
    for(i=0;i<count1;i+=2){
	/** coordinate **/
	Tcl_SetListObj(coordlist, 0, NULL);
	TCL_APPEND(coordlist, Double, interpolate_amp(n, va[i+1], a, t));
	TCL_APPEND(coordlist, Double, va[i+1]);
	TCL_APPEND(coordlist, Double, gain);
	TCL_APPEND(coordlist, Double, va[i+1]);
	TCL_APPEND(coordlist, Double, gain);
	TCL_APPEND(coordlist, Double, va[i]);
	TCL_APPEND(coordlist, Double, interpolate_amp(n, va[i], a, t));
	TCL_APPEND(coordlist, Double, va[i]);
	for(j=0;j<n;j++){
	    if( (va[i+1] - t[j])*(t[j] - va[i]) > 0.0 ){
		TCL_APPEND(coordlist, Double, a[j]);
		TCL_APPEND(coordlist, Double, t[j]);
	    }
	}
	sprintf(com, "%s marker create polygon -coor $coordlist"
		"  -fill #%06x -name red%.3g_%d", g, col1, x, i);
	Tcl_Eval(interp, com);
    }
    if(col2 == TRANSPARENT) return;
    /***** minus side *****/
    /** threshold points **/
    gain = x - threshold;
    count1 = search_threshold_minus(n, a, t, va, gain);
    /** plot **/
    for(i=0;i<count1;i+=2){
	Tcl_SetListObj(coordlist, 0, NULL);
	TCL_APPEND(coordlist, Double, interpolate_amp(n, va[i+1], a, t));
	TCL_APPEND(coordlist, Double, va[i+1]);
	TCL_APPEND(coordlist, Double, gain);
	TCL_APPEND(coordlist, Double, va[i+1]);
	TCL_APPEND(coordlist, Double, gain);
	TCL_APPEND(coordlist, Double, va[i]);
	TCL_APPEND(coordlist, Double, interpolate_amp(n, va[i], a, t));
	TCL_APPEND(coordlist, Double, va[i]);
	for(j=0;j<n;j++){
	    if( (va[i+1] - t[j])*(t[j] - va[i]) > 0.0 ){
		TCL_APPEND(coordlist, Double, a[j]);
		TCL_APPEND(coordlist, Double, t[j]);
	    }
	}
	sprintf(com, "%s marker create polygon -coor $coordlist"
		"  -fill #%06x -name blue%.3g_%d", g, col2, x, i);
	Tcl_Eval(interp, com);
    }
}

/********************************************************************
Output one trace for GraphCanvas
--------------------------------------------------------------------
 mode        : drawing mode
 n           : dimension of a and t 
 a           : amplitude array (waveform)
 t           : time array
 x           : trace x-position (offset distacne)
 clip        : clip amplitude
 threshold   : threshold level for VA
********************************************************************/
void draw_graph(Tcl_Interp *interp, int mode, int n, float *a, float *t,
		double x, float clip, float threshold)
{
    if(mode<10){
	/* Wiggle/VA */
	amp_clip_float(a, n, clip);
	switch(mode){
	    case WIGGLE_MODE:
		draw_graph_wiggle(interp, n, a, t, x);
		break;
	    case VA_MODE:
		draw_graph_wiggle(interp, n, a, t, x);
		draw_graph_VA(interp, n, a, t, x, threshold, 0x0, TRANSPARENT);
		break;
	    case VA_WITHOUT_WIGGLE:
		draw_graph_VA(interp, n, a, t, x, threshold, 0x0, TRANSPARENT);
		break;
	    case VA_REDBLUE_MODE:
		draw_graph_wiggle(interp, n, a, t, x);
		draw_graph_VA(interp, n, a, t, x, threshold,
			      0xff0000, 0x0000ff);
		break;
	    case VA_WO_WIGGLE_RB:
		draw_graph_VA(interp, n, a, t, x, threshold,
			      0xff0000, 0x0000ff);
		break;
	}
    } else {
	/* VD */
    }
}
