#include <stdio.h>
#include <string.h>
#include <math.h>

#include "pasteup.h"

/********************************************************************
wiggle 
--------------------------------------------------------------------
Inputs:
 im         : Image pointer
 n          : Dimension of a and t 
 a          : Amplitude array (waveform)
 t          : Time array
 t_axis     : Direction of time axis (1 downward-1 upward)
 amp_cutoff : Clip amplitude
 tmin       : minimum time to show
 tmax       : maximum time to show
 x          : trace x-position (offset distance in canvas coordinate)
 height     : Height of the canvas (pixel)
 top_offset : top margin (pixel)
********************************************************************/
static void draw_image_wiggle(gdImagePtr im, int n, float *a, float *t,
			      double x, int t_axis, float tmin, float tmax, 
			      double height, double top_offset, int color[])
{
    /********** Local Variables *********/
    register int i;
    register float c;
    float *tg, *ag;

    /********** Main *********/
    if( NULL == (tg = (float *)malloc(n * sizeof(float))) ||
	NULL == (ag = (float *)malloc(n * sizeof(float))) ) {
	PASTEUP_ERROR;
    }

    c = height/(tmax - tmin);
    if(t_axis>0) {
	for(i=0;i<n;i++){
	    tg[i] = rintf( (t[i] - tmin)*c + top_offset );
	    ag[i] = rintf(a[i] + x);
	}
    } else {
	for(i=0;i<n;i++){
	    tg[i] = rintf( height - (t[i] - tmin)*c + top_offset);
	    ag[i] = rintf(a[i] + x);
	}
    }
    
    /** Draw Wiggle **/
    for(i=0;i<n-1;i++){
	gdImageLine(im, ag[i], tg[i], ag[i+1], tg[i+1], color[BLACK]);
    }
}

/********************************************************************
Variable Density with red-blue color
--------------------------------------------------------------------
Inputs:
 n          : Dimension of a and t 
 a          : Amplitude array (waveform)
 t          : Time array
 t_axis     : Direction of time axis (1 downward-1 upward)
 amp_step   : Amplitude step to change color (nomeanings in this function)
 tmin : minimum time to show
 tmax   : maximum time to show
 sampling   : sampling frequency (Hz)
 x          : trace x-position (offset distance in canvas coordinate (pixel))
 height     : Height of the canvas (pixel)
 top_offset : Height of the top margin region (pixel)
 lw         : line width
********************************************************************/
static void draw_image_VD(gdImagePtr im, int n, float *a, float *t, int t_axis,
			  int amp_step, float tmin, float tmax, float sampling,
			  double x, double height, double top_offset,
			  int lw, int color[])
{
    /********** Local Variables *********/
    register int i, plotx1, plotx2;
    register float c;

    /******************** Main *******************/
    if(n<1)return; /* too short trace */

    /** time-axis coefficient **/
    c = height/(tmax - tmin);

    /***** x-coordinate *****/
    plotx1 = rint(x - (lw - 1)/2.0);
    plotx2 = rint(x + (lw - 1)/2.0);

    if ( c > sampling ) {
	int code;
	/********** if dt > 1 pixel **********/
	if(t_axis>0){
	    for(i=0;i<height;i++) {
		code = amp2index(interpolate_amp(n, i/c + tmin, a, t),
				 amp_step);
		gdImageLine(im, plotx1, i + top_offset, plotx2, i + top_offset,
			    color[code]);
	    }
	} else {
	    for(i=0;i<height;i++) {
		code = amp2index(interpolate_amp(n, i/c + tmin, a, t),
				 amp_step);
		gdImageLine(im, plotx1, top_offset + height - i, plotx2,
			    top_offset + height - i, color[code]);
	    }
	}
    } else {
	/********** if dt <= 1 pixel **********/
	register int tg, tg_prev, m;
	register float ag;

	if(t_axis>0){
	    m = 1;
	    ag = a[0];
	    tg_prev = rintf( (t[0] - tmin)*c + top_offset );
	    for(i=1;i<n;i++){
		tg = rintf( (t[i] - tmin)*c + top_offset );
		if (tg != tg_prev) {
		    gdImageLine(im, plotx1, tg_prev, plotx2, tg_prev,
				color[amp2index(ag/m, amp_step)]);
		    tg_prev = tg;
		    ag = a[i];
		    m = 1;
		} else {
		    ag += a[i];
		    m++;
		}
	    }
	    gdImageLine(im, plotx1, tg_prev, plotx2, tg_prev,
			color[amp2index(ag/m, amp_step)]);
	} else {
	    m = 1;
	    ag = a[0];
	    tg_prev = rintf( height - (t[0] - tmin)*c + top_offset );
	    for(i=1;i<n;i++){
		tg = rintf( height - (t[i] - tmin)*c + top_offset );
		if ( tg != tg_prev) {
		    gdImageLine(im, plotx1, tg_prev, plotx2, tg_prev,
				color[amp2index(ag/m, amp_step)]);
		    tg_prev = tg;
		    ag = a[i];
		    m = 1;
		} else {
		    ag += a[i];
		    m++;
		}
	    }
	    gdImageLine(im, plotx1, tg_prev, plotx2, tg_prev,
			color[amp2index(ag/m, amp_step)]);
	}
    }
}

/********************************************************************
Variable Area with Red&Blue color
--------------------------------------------------------------------
Input:
 n          : Dimension of a and t 
 a          : Amplitude array (waveform)
 t          : Time array
 mode       : Output mode (Vector, image)
 t_axis     : Direction of time axis (1 downward-1 upward)
 amp_step   : Amplitude step to change color (nomeanings in this function)
 tmin       : minimum time to show
 tmax       : maximum time to show
 x          : offset distance in canvas coordinate (pixel)
 height     : Height of the canvas (pixel)
 top_offset : Height of the top margin region (pixel)
 col1       : plus side color
 col2       : minus side color
********************************************************************/
static void draw_image_VA(gdImagePtr im, int n, float *a, float *t, int t_axis,
			  int amp_step, float tmin, float tmax, double x,
			  double height, double top_offset, int col1, int col2)
{
    /********** Local Variables *********/
    register int i;
    register float c;
    float *tg, tmp;
    float atmp;
    float gain, mgain;

    /******************** Main *******************/
    if(n<2) return; /* Too short trace */

    if( NULL == (tg = (float *)malloc(n * sizeof(float))) ){
	PASTEUP_ERROR;
    }

    /** time-axis coefficient **/
    c = height/(tmax - tmin);

    /***** Convert time into graph coordinates *****/
    if(t_axis>0){
	for(i=0;i<n;i++) {
	    a[i] = rintf(a[i] + x);
	    tg[i]=(t[i] - tmin)*c + top_offset;
	}
    } else {
	for(i=0;i<n;i++) {
	    a[i] = rintf(a[i] + x);
	    tg[i] = height - (t[i] - tmin)*c + top_offset;
	}
    }
	       
    /***** variable area *****/
    gain = amp_step + x;
    mgain = -amp_step + x;
    tmp = tg[0];
    if(t_axis>0) { /* t-axis downward */
	while(tmp <= tg[n-1]){
	    atmp = interpolate_amp(n, tmp, a, tg);
	    if(atmp>gain){
		gdImageLine(im, gain, tmp, atmp, tmp, col1);
	    } else if(col2 != TRANSPARENT && atmp < mgain) {
		gdImageLine(im, mgain, tmp, atmp, tmp, col2);
	    }
	    tmp += 1.0;
	}
    } else { /* t-axis upward */
	while(tmp >= tg[n-1]){
	    atmp = interpolate_amp(n, tmp, a, tg);
	    if(atmp > gain){
		gdImageLine(im, gain, tmp, atmp, tmp, col1);
	    } else if(col2 != TRANSPARENT && atmp < mgain) {
		gdImageLine(im, mgain, tmp, atmp, tmp, col2);
	    }
	    tmp -= 1.0;
	}
    }
}

/********************************************************************
Output one trace
--------------------------------------------------------------------
 n          : Dimension of a and t
 a          : Amplitude array (waveform)
 t          : Time array
 mode       : Output mode (Vector, image)
 t_axis     : Direction of time axis (1 downward-1 upward)
 amp_step   : Amplitude step to change color (nomeanings in this function)
 amp_cutoff : Clip amplitude
 tmin       : minimum time to show
 tmax       : maximum time to show
 sampling   : sampling frequency (Hz)
 x          : trace x-position (offset distance in canvas coordinate (pixel))
 height     : Height of the canvas (pixel)
 top_offset : Height of the top margin region (pixel)
 lw         : line width
 color      : colors
********************************************************************/
void draw_image(gdImagePtr im, int n, float *a, float *t, int mode,
		int t_axis, int amp_step, int amp_cutoff,
		float tmin, float tmax, float sampling, double x,
		double height, double top_offset, int lw, int color[])
{
    if(mode < 10){
	/* Wiggle/VA */
	amp_clip(a, n, amp_cutoff);
	switch(mode){
	    case WIGGLE_MODE:
		draw_image_wiggle(im, n, a, t, x, t_axis, tmin, tmax,
				  height, top_offset, color);
		break;
	    case VA_MODE:
		draw_image_wiggle(im, n, a, t, x, t_axis, tmin, tmax,
				  height, top_offset, color);
		draw_image_VA(im, n, a, t, t_axis, amp_step, tmin, tmax, x,
			      height, top_offset, color[BLACK], TRANSPARENT);
		break;
	    case VA_WITHOUT_WIGGLE:
		draw_image_VA(im, n, a, t, t_axis, amp_step, tmin, tmax, x,
			      height, top_offset, color[BLACK], TRANSPARENT);
		break;
	    case VA_REDBLUE_MODE:
		draw_image_wiggle(im, n, a, t, x, t_axis, tmin, tmax,
				  height, top_offset, color);
		draw_image_VA(im, n, a, t, t_axis, amp_step, tmin, tmax, x,
			      height, top_offset, color[RED], color[BLUE]);
		break;
	    case VA_WO_WIGGLE_RB:
		draw_image_VA(im, n, a, t, t_axis, amp_step, tmin, tmax, x,
			      height, top_offset, color[RED], color[BLUE]);
		break;
	}
    } else {
	/* VD */
	draw_image_VD(im, n, a, t, t_axis, amp_step, tmin, tmax,
		      sampling, x, height, top_offset, lw, color);
    }
}
