#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>

#include "pasteup.h"

/********************************************************************
wiggle 
--------------------------------------------------------------------
Inputs:
 n           : Dimension of a and t 
 a           : Amplitude array (waveform)
 t           : Time array
 t_axis      : Direction of time axis (1 downward-1 upward)
 tmin        : minimum time to show
 tmax        : maximum time to show
 x           : trace x-position
 height      : Height of the canvas (pixel)
 top_offset  : top margin (pixel)
********************************************************************/
static void draw_vector_wiggle(Tcl_Interp *interp, int n, float *a, float *t,
			       int t_axis, float tmin, float tmax,
			       double x, double height, double top_offset)
{
    /********** Local Variables *********/
    register int i;
    register float c;
    double line_width;
    Tcl_Obj* coordlist;
    char com[256];

    /*************** Main **************/
    c = height/(tmax - tmin);       /* time-axis coefficient */
    
    /** make list **/
    coordlist = Tcl_NewListObj(0, NULL);
    Tcl_SetVar2Ex(interp, "coordlist", NULL, coordlist, 0);
    if(t_axis>0){
	for(i=0;i<n;i++){
	    TCL_APPEND(coordlist, Double, a[i] + x);
	    TCL_APPEND(coordlist, Double, (t[i] - tmin)*c + top_offset);
	}
    } else {
	for(i=0;i<n;i++){
	    TCL_APPEND(coordlist, Double, a[i] + x);
	    TCL_APPEND(coordlist, Double, height - (t[i]-tmin)*c + top_offset);
	}
    }

    /** draw !! **/
    TCL_GET(Double, gbl, "line_width", line_width);
    sprintf(com, "%s create line $coordlist -width %g -tags wave",
	    TCL_GETSTRING(wk, "w,canvas,c"), line_width/5);
    Tcl_Eval(interp, com);
}

/********************************************************************
Variable Density with red-blue color
--------------------------------------------------------------------
Inputs:
 n          : Dimension of a and t 
 a          : Amplitude array (waveform)
 t          : Time array
 t_axis     : Direction of time axis (1 downward-1 upward)
 amp_step   : Amplitude step to change color (nomeanings in this function)
 tmin       : minimum time to show
 tmax       : maximum time to show
 x          : offset distance in canvas coordinate (pixel)
 height     : Height of the canvas (pixel)
 top_offset : top margin (pixel)
 color      : color code table
********************************************************************/
static void draw_vector_VD(Tcl_Interp *interp,
			   int n, float *a, float *t, int t_axis,
			   int amp_step, float tmin, float tmax, double x,
			   double height, double top_offset, int *color)
{
    /********** Local Variables *********/
    register int i, count1, tmp_code;
    register float c;
    float *tg;
    int *code;

    Tcl_Obj* coordlist;
    char com[128];
    int line_width;

    /******************** Main *******************/
    if(n<1)return; /* too short trace */

    if( NULL == (tg = (float *)malloc(n * sizeof(float))) ||
	NULL == (code = (int *)malloc(n * sizeof(int))) ) {
	PASTEUP_ERROR;
    }

    /** time-axis coefficient **/
    c = height/(tmax - tmin);
    count1 = n;
    code[0] = -1; /*** code[0] is dummy to compare ***/
    n = 1;
	       
    /*** Convert time into graph coordinate ***/
    if(t_axis>0){
	for(i=0;i<count1;i++){
	    tmp_code = color[amp2index(a[i], amp_step)];
	    if(code[n-1] != tmp_code){
		tg[n] = (t[i] - tmin)*c + top_offset;
		code[n++] = tmp_code; 
	    }
	}
	/*** The last point ***/
	tg[n] = (t[count1-1] - tmin)*c + top_offset;
    } else {
	for(i=0;i<count1;i++){
	    tmp_code = color[amp2index(a[i], amp_step)];
	    if(code[n-1] != tmp_code){
		tg[n] = height - (t[i] - tmin)*c + top_offset;
		code[n++] = tmp_code;
	    }
	}
	/*** The last point ***/
	tg[n] = height + top_offset - (t[count1-1] - tmin)*c;
    }

    /***** Draw *****/
    TCL_GET(Int, gbl, "line_width", line_width);
    coordlist = Tcl_NewListObj(0, NULL);
    Tcl_SetVar2Ex(interp, "coordlist", NULL, coordlist, 0);
    for(i=1;i<n;i++){
	Tcl_SetListObj(coordlist, 0, NULL);
	TCL_APPEND(coordlist, Double, x);
	TCL_APPEND(coordlist, Double, tg[i]);
	TCL_APPEND(coordlist, Double, x);
	TCL_APPEND(coordlist, Double, tg[i+1]);
	sprintf(com,
		"%s create line $coordlist -width %d -tags wave -fill #%06x",
		TCL_GETSTRING(wk, "w,canvas,c"), line_width, code[i]);
	Tcl_Eval(interp, com);
    }
}

/********************************************************************
Variable Area
--------------------------------------------------------------------
Input:
 n          : Dimension of a and t
 a          : Amplitude array (waveform)
 t          : Time array
 t_axis     : Direction of time axis (1 downward-1 upward)
 amp_step   : Amplitude step to change color (nomeanings in this function)
 tmin       : minimum time to show
 tmax       : maximum time to show
 x          : offset distance in canvas coordinate (pixel)
 height     : Height of the canvas (pixel)
 top_offset : top margin (pixel)
 col1       : plus side color
 col2       : minus side color
********************************************************************/
static void draw_vector_VA(Tcl_Interp *interp, int n, float *a, float *t,
			   int t_axis, int amp_step, float tmin, float tmax,
			   double x, double height, double top_offset,
			   int col1, int col2)
{
    /********** Local Variables *********/
    register int count1, i, j;
    register float c;
    float *tg, *va;
    float gain;
    Tcl_Obj* coordlist;
    char com[256];

    /******************** Main *******************/
    if(n<2) return; /* Too short trace */

    if( NULL == (tg = (float *)malloc(n * sizeof(float))) ||
	NULL == (va = (float *)malloc(n * sizeof(float))) ) {
	PASTEUP_ERROR;
    }

    /** time-axis coefficient **/
    c = height/(tmax - tmin);

    /***** Convert time into graph coordinates *****/
    if(t_axis>0){
	for(i=0;i<n;i++) {
	    a[i] += x;
	    tg[i]=(t[i] - tmin)*c + top_offset;
	}
    } else {
	for(i=0;i<n;i++) {
	    a[i] += x;
	    tg[i] = height - (t[i] - tmin)*c + top_offset;
	}
    }
	       
    /***** RED : Variable area *****/
    coordlist = Tcl_NewListObj(0, NULL);
    Tcl_SetVar2Ex(interp, "coordlist", NULL, coordlist, 0);

    /** threshold points **/
    gain = amp_step + x;
    count1 = search_threshold(n, a, tg, va, gain);
    /** plot **/
    for(i=0;i<count1;i+=2){
	Tcl_SetListObj(coordlist, 0, NULL);
	TCL_APPEND(coordlist, Double, interpolate_amp(n, va[i+1], a, tg));
	TCL_APPEND(coordlist, Double, va[i+1]);
	TCL_APPEND(coordlist, Double, gain);
	TCL_APPEND(coordlist, Double, va[i+1]);
	TCL_APPEND(coordlist, Double, gain);
	TCL_APPEND(coordlist, Double, va[i]);
	TCL_APPEND(coordlist, Double, interpolate_amp(n, va[i], a, tg));
	TCL_APPEND(coordlist, Double, va[i]);
	for(j=0;j<n;j++){
	    if( (va[i+1] - tg[j])*(tg[j] - va[i]) > 0.0 ){
		TCL_APPEND(coordlist, Double, a[j]);
		TCL_APPEND(coordlist, Double, tg[j]);
	    }
	}
	sprintf(com, "%s create polygon $coordlist -tags wave -fill #%06x",
		TCL_GETSTRING(wk, "w,canvas,c"), col1);
	Tcl_Eval(interp, com);

    }
    if(col2 != TRANSPARENT) {
	/***** BLUE : Variable area *****/
	/** threshold points **/
	gain = -amp_step + x;
	count1 = search_threshold_minus(n, a, tg, va, gain);
	/** plot **/
	for(i=0;i<count1;i+=2){
	    Tcl_SetListObj(coordlist, 0, NULL);
	    TCL_APPEND(coordlist, Double, interpolate_amp(n, va[i+1], a, tg));
	    TCL_APPEND(coordlist, Double, va[i+1]);
	    TCL_APPEND(coordlist, Double, gain);
	    TCL_APPEND(coordlist, Double, va[i+1]);
	    TCL_APPEND(coordlist, Double, gain);
	    TCL_APPEND(coordlist, Double, va[i]);
	    TCL_APPEND(coordlist, Double, interpolate_amp(n, va[i], a, tg));
	    TCL_APPEND(coordlist, Double, va[i]);
	    for(j=0;j<n;j++){
		if( (va[i+1] - tg[j])*(tg[j] - va[i]) > 0.0 ){
		    TCL_APPEND(coordlist, Double, a[j]);
		    TCL_APPEND(coordlist, Double, tg[j]);
		}
	    }
	    sprintf(com, "%s create polygon $coordlist -tags wave -fill #%06x",
		    TCL_GETSTRING(wk, "w,canvas,c"), col2);
	    Tcl_Eval(interp, com);
	}
    }
    free(tg);
    free(va);
}

/********************************************************************
Output one trace
--------------------------------------------------------------------
 a          : Amplitude array (waveform)
 t          : Time array
 n          : Dimension of a and t 
 mode       : Output mode (Vector, image)
 t_axis     : Direction of time axis (1 downward-1 upward)
 amp_step   : Amplitude step to change color (nomeanings in this function)
 amp_cutoff : Clip amplitude
 tmin       : minimum time to show
 tmax       : maximum time to show
 x          : trace x-position in NormalCanvas widget
 height     : Height of the canvas (pixel)
 top_offset : top margin (pixel)
 color      : color code table
********************************************************************/
void draw_vector(Tcl_Interp *interp, float *a, float *t, int n,
		 int mode, int t_axis, int amp_step, int amp_cutoff,
		 float tmin, float tmax, double x, double height,
		 double top_offset, int *color)
{
    if(mode < 10){
	/* Wiggle/VA */
	amp_clip(a, n, amp_cutoff);
	switch(mode){
	    case VA_MODE:
		draw_vector_wiggle(interp, n, a, t, t_axis, tmin, tmax,
				   x, height, top_offset);
		draw_vector_VA(interp, n, a, t, t_axis, amp_step, tmin, tmax,
			       x, height, top_offset, 0x0, TRANSPARENT);
		break;
	    case VA_WITHOUT_WIGGLE:
		draw_vector_VA(interp, n, a, t, t_axis, amp_step, tmin, tmax,
			       x, height, top_offset, 0x0, TRANSPARENT);
		break;
	    case VA_REDBLUE_MODE:
		draw_vector_wiggle(interp, n, a, t, t_axis, tmin, tmax,
				   x, height, top_offset);
		draw_vector_VA(interp, n, a, t, t_axis, amp_step, tmin, tmax,
			       x, height, top_offset, 0xff0000, 0x0000ff);
		break;
	    case VA_WO_WIGGLE_RB:
		draw_vector_VA(interp, n, a, t, t_axis, amp_step, tmin, tmax,
			       x, height, top_offset, 0xff0000, 0x0000ff);
		break;
	    case WIGGLE_MODE:
		draw_vector_wiggle(interp, n, a, t, t_axis, tmin, tmax,
				   x, height, top_offset);
		break;
	}
    } else {
	/* VD */
	draw_vector_VD(interp, n, a, t, t_axis, amp_step, tmin, tmax,
		       x, height, top_offset, color);
    }

}
