#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <tcl.h>

#include "pasteup.h"

/********************************************************************

Get the maximum amplitude

********************************************************************/
int pasteup_getampProc(
		       ClientData clientData, /* not used                */
		       Tcl_Interp *interp,    /* current tcl interpreter */
		       int objc,              /* number of arguments     */
		       Tcl_Obj* CONST objv[]  /* arguments               */
		       )
{
    /********** Variables *********/
    /** arguments **/
    double start_time, end_time;
    double left, right;

    /***** Tcl interpreter *****/
    char *data_filename; /*** waveform filename ***/
    int data_format;         /** waveform data format (SU, SEG-Y...)**/
    int data_byte;           /** size of one binary sample **/
    int wave_density;        /*** trace display freq ***/
    double wave_dist;        /*** trace display interval in km ***/
    int amp_correct;         /*** amp correction mode ***/
    double geom_L, geom_a;   /** parameters for geometric spreading correct.**/
    double agc_period;       /*** AGC window (sec) ***/
    
    /**** etc. ****/
    int i, j, n;
    double kika_coef;
    double prev_position;

    float *t, *a, t_org;
    float amp, local_max, max_amp;

    FILE *fp;
    
    /**********************************************/
    /******************** Main ********************/
    /**********************************************/
    
    void usage () {
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n getamp x1 x2 t1 t2\n",
	     "",
	     (char *)NULL);
    }

    /********** Check arguments **********/
    if(objc != 5){
	usage();
	return TCL_ERROR;
    }

    if(
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[1], &left) ||
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[2], &right) ||
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[3], &start_time) ||
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[4], &end_time) ) {
	usage();
	return TCL_ERROR;
    }

    /*************** Read global variables ***************/
    data_filename = TCL_GETSTRING(file_name, "wave");
    TCL_GET(Int, gbl, "data,type", data_format);
    TCL_GET(Int, gbl, "data,byte", data_byte);
    TCL_GET(Int, paste, "wave_density", wave_density);
    TCL_GET(Double, paste, "wave_distance", wave_dist);
    TCL_GET(Double, paste, "amp,agc", agc_period);
    TCL_GET(Int, paste, "amp,correct", amp_correct);
    TCL_GET(Double, paste, "amp,geometric,standard", geom_L);
    TCL_GET(Double, paste, "amp,geometric,power", geom_a);

    /********** Check left and right **********/
    if(left > right) {
	/** swap left and right **/
	amp = left;  /* amp is temporary */
	left = right;
	right = amp;
    }

    /************************************************************/
    /************************ Make cache ************************/
    /************************************************************/
    
    total_shot = make_cache_data(interp, start_time, end_time);

    /************************************************************/
    /*********************** Preparations ***********************/
    /************************************************************/

    /*** memory allocation ***/
    if(NULL == (t = (float *)malloc(maxsample * sizeof(float))) ||
       NULL == (a = (float *)malloc(maxsample * sizeof(float))) ){
	fprintf(stderr, "Error: Memory allocation failed.\n");
	return TCL_ERROR;
    }
    
    amp = 1.0;
    max_amp = 0.0;
    prev_position = gbl_rh[0].x - wave_dist*2.0;

    /************************************************************/
    /***************** Search max amp by cache ******************/
    /************************************************************/

    for(i=0;i<total_shot;i+=wave_density) {
	if( (gbl_rh[i].x >= left) && (gbl_rh[i].x <=right)
	    && (wave_density <= 0.0 ||
		fabs(prev_position - gbl_rh[i].x) >= wave_dist)
	    ) {
	    prev_position = gbl_rh[i].x;
		
	    /*** Geometric spreading ***/
	    kika_coef = pow(fabs(gbl_rh[i].x)/geom_L, geom_a);
		
	    /*** make a trace ***/
	    n = form_trace(gbl_a[i], gbl_t[i], gbl_n[i],
			   start_time, end_time, gbl_rh[i].sampling,
			   amp, amp_correct, agc_period, kika_coef, a, t);
	    /*** search max ***/
	    local_max = 0.0;
	    for(j=0;j<n;j++)
		if(fabs(a[j]) > local_max) local_max = fabs(a[j]);
	    if(local_max > max_amp) max_amp = local_max;
	}
    }

    Tcl_SetObjResult(interp, Tcl_NewDoubleObj(max_amp));
    free(t);
    free(a);
    return TCL_OK;
}

