#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <tcl.h>

#include "pasteup.h"

#define RIGHT 1
#define LEFT  0

/********************************************************************

Read wave-data and draw them in the interpolated mode

********************************************************************/
int pasteup_getinterpProc(
			  ClientData clientData, /* not used                */
			  Tcl_Interp *interp,    /* current tcl interpreter */
			  int objc,              /* number of arguments     */
			  Tcl_Obj* CONST objv[]  /* arguments               */
			  )
{
    /********** Variables *********/
    /***** Variables for tcl interaction *****/
    char *data_filename; /*** waveform filename ***/
    int data_byte;           /** size of one binary sample **/
    int mode;                /*** display style (wiggle, va...) ***/
    double amp;              /*** scale factor ***/
    int amp_step;            /*** threshold level ***/
    double start_time, end_time;
    double left, right;
    int ind;
    int wave_density;        /*** trace display freq ***/
    double graph_width, graph_height;
    int t_axis, amp_correct, amp_class, amp_cutoff;
    double geom_L, geom_a;   /** parameters for geometric spreading correct.**/
    double agc_period;       /*** AGC window (sec) ***/
    int canvas;              /*** canvas mode   ***/
    
    int image_format;        /*** png(+1), jpeg(-1)  ***/
    int ncol;                /*** number of colors ***/

    int xaxis;               /*** header for xaxis ***/
    int xaxis_direc;         /*** xaxis direction (1:normal, -1:reverse) ***/

    /**** etc. ****/
    FILE *fp;
    
    int i, right_index, ix, j, interp_n;
    double x, dx, ratio1, ratio2, left_amp, right_amp, dx_max;
    double kika_coef;

    int n_left, n_right;
    float **a, **t, *interp_a;
    float t_org[2];           /** t[0]:left, t[1]:right **/
    int count[2];

    /*** For GD ***/
    gdImagePtr im;
    int color[BLACK+1];
    int imagesize;
    char *image;
    
    /**********************************************/
    /******************** Main ********************/
    /**********************************************/
    
    /********** Get Option **********/
    Tcl_GetIntFromObj(interp, objv[1], &mode);
    Tcl_GetIntFromObj(interp, objv[2], &image_format);  /** jpeg/png **/
    Tcl_GetDoubleFromObj(interp, objv[3], &amp);        /** scale factor **/

    /*************** Read global variables ***************/
    data_filename = TCL_GETSTRING(file_name, "wave");
    TCL_GET(Int, gbl, "data,byte", data_byte);
    TCL_GET(Double, paste, "left", left);
    TCL_GET(Double, paste, "right", right);
    TCL_GET(Double, paste, "top", start_time);
    TCL_GET(Double, paste, "bottom", end_time);
    TCL_GET(Int, paste, "canvas,mode", canvas);
    if (canvas == 2) {
	/** graph canvas **/
	TCL_GET(Double, paste, "graph,dx",dx);
	graph_width = (right - left) / dx;
	while(graph_height > 2500) { graph_height /= 2.0; }
	TCL_GET(Double, paste, "graph_width", dx);
	graph_width = (dx > graph_width) ? dx : graph_width;

	TCL_GET(Double, paste, "graph,dt",dx);
 	graph_height = (end_time - start_time) / dx;
	while(graph_width > 4500) { graph_width /= 2.0; }
	TCL_GET(Double, paste, "graph_height", dx);
	graph_height = (dx > graph_height) ? dx : graph_height;
    } else {
	/** normal canvas **/
	TCL_GET(Double, paste, "graph_height", graph_height);
	TCL_GET(Double, paste, "graph_width", graph_width);
    }
    TCL_GET(Int, gbl, "t_axis", t_axis);
    TCL_GET(Int, paste, "wave_density", wave_density);

    TCL_GET(Int, paste, "amp,threshold", amp_step);
    TCL_GET(Int, paste, "amp,correct", amp_correct);
    TCL_GET(Int, paste, "amp,cutoff", amp_cutoff);
    TCL_GET(Int, paste, "amp,class", amp_class);
    TCL_GET(Double, paste, "amp,agc", agc_period);

    TCL_GET(Double, paste, "amp,geometric,standard", geom_L);
    TCL_GET(Double, paste, "amp,geometric,power", geom_a);
    TCL_GET(Int, paste, "ncolors", ncol);
    
    TCL_GET(Int, gbl, "data,xaxis", xaxis);
    TCL_GET(Int, gbl, "data,xaxis_direction", xaxis_direc);
    TCL_GET(Double, paste, "interpolate,threshold", dx_max);

    /************************************************************/
    /************************ Make cache ************************/
    /************************************************************/
    
    total_shot = make_cache_data(interp, start_time, end_time);

    /************************************************************/
    /*********************** Preparations ***********************/
    /************************************************************/

    /*** amplification ***/
    amp *= compute_amp_coefficient(mode);
    amp_step = compute_amp_step(data_byte, mode, amp_step);
    
    /*** memory allocation ***/
    if(NULL == (t = fmalloc2(maxsample, 2)) ||
       NULL == (a = fmalloc2(maxsample, 2)) ||
       NULL == (interp_a = (float *)malloc(maxsample * sizeof(float)))) {
	fprintf(stderr, "Error: Memory allocation failed.\n");
	return TCL_ERROR;
    }

    /********** for Image file **********/
    /* Allocate the image */
    im = gdImageCreateTrueColor(graph_width+1, graph_height+1);
    /* Allocate colors. */
    declare_colors(interp, im, color, mode, ncol);
    /* Clear canvas */
    gdImageFilledRectangle(im, 0, 0, graph_width+1, graph_height+1,
			   color[WHITE]);

    /************************************************************/
    /***************** Draw by using cache data *****************/
    /************************************************************/
    
    right_index = 0;
    for(ix=1;ix<graph_width;ix++){

	/***** Canvas coordinate (ix) -> real coordinate (x) *****/
	x = ix * (right - left) / graph_width + left;

	/***** Search right index *****/
	while(gbl_rh[right_index].x < x) {
	    if(++right_index >= total_shot) break;
	}
	/** outside data range **/
	if(right_index>=total_shot)break;
	if(right_index <= 0)continue;

	/***** Interpolation *****/
	dx = gbl_rh[right_index].x - gbl_rh[right_index - 1].x;
	if(dx > dx_max) continue;
	ratio1 = (x - gbl_rh[right_index - 1].x) / dx;
	ratio2 = (gbl_rh[right_index].x - x) / dx;
	ind = (ratio1>ratio2) ? RIGHT : LEFT;

	/***** make left and right traces *****/
	kika_coef =pow(fabs(gbl_rh[right_index-1].x)/geom_L,geom_a);
	n_left = form_trace(gbl_a[right_index-1], gbl_t[right_index-1],
			    gbl_n[right_index-1],
			    start_time, end_time,
			    gbl_rh[right_index-1].sampling,
			    amp, amp_correct,
			    agc_period, kika_coef, a[LEFT], t[LEFT]);
	kika_coef =pow(fabs(gbl_rh[right_index].x)/geom_L,geom_a);
	n_right = form_trace(gbl_a[right_index], gbl_t[right_index],
			     gbl_n[right_index],
			     start_time, end_time,
			     gbl_rh[right_index].sampling,
			     amp, amp_correct,
			     agc_period, kika_coef, a[RIGHT], t[RIGHT]);
	interp_n = (n_left > n_right) ? n_right : n_left;
	if(interp_n>maxsample)interp_n=maxsample;
	for(j=0;j<interp_n;j++){
	    left_amp = interpolate_amp(n_left, t[ind][j],a[LEFT], t[LEFT]);
	    right_amp = interpolate_amp(n_right, t[ind][j],
					a[RIGHT], t[RIGHT]);
	    interp_a[j] = right_amp * ratio1 + left_amp * ratio2;
	}
	if(xaxis_direc == 0) {
	    draw_image(im, interp_n, interp_a, t[ind], mode, t_axis,
		       amp_step, amp_cutoff,
		       start_time, end_time, gbl_rh[right_index].sampling,
		       ix, graph_height, 0, 1, color);
	} else {
	    draw_image(im, interp_n, interp_a, t[ind], mode, t_axis,
		       amp_step, amp_cutoff,
		       start_time, end_time, gbl_rh[right_index].sampling,
		       graph_width - ix, graph_height, 0, 1, color);
	}
	fflush(stderr);
    }

    /********** Make PNG/JPEG file **********/
    if(image_format > 0) image = (char *)gdImagePngPtr(im, &imagesize);
    else                 image = (char *)gdImageJpegPtr(im, &imagesize, 90);
    Tcl_SetObjResult(interp, Tcl_NewByteArrayObj(image, imagesize));
    gdFree(image);
    gdImageDestroy(im);
    ffree2(t);
    ffree2(a);
    free(interp_a);
    return TCL_OK;
}

