#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>

#include "pasteup.h"
#include "tcl.h"

/********************************************************************

Read all the record header info and make a cache.
This file is a part of Tcl/Tk extension package, libpasteup.so.

********************************************************************/

int pasteup_getrhProc(
		      ClientData clientData, /* not used                */
		      Tcl_Interp *interp,    /* current tcl interpreter */
		      int objc,              /* number of arguments     */
		      Tcl_Obj* CONST objv[]  /* arguments               */
		      )
{
    /********** Variables *********/
    char *opt;
    
    /***** for Tcl interpreter *****/
    double left, right;    /** left and right end                       **/
    double lon, lat;       /** OBS lat, lon (for THK)                   **/
    int hw_shot;           /** header word name for shot number         **/
    int hw_site;           /** header word name for site number         **/
    double start_time, end_time;
    
    /**** etc ****/
    int i, j;
    double dist, min_dist;
    Tcl_Obj* list;

    /******************** Main ********************/
    
    void usage () {
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n getrh option listName\n",
	     "   option is one of followings.\n",
	     "       makecache : Update all the record header info\n",
	     "       minmax    : return min and max offset",
	     "       nearest   : search nearest trace\n",
	     "       range     : search traces in the display range\n",
	     "       shotpos   : return x-position of the specified shot number", 
	     "",
	     (char *)NULL);
    }

    /********** Check arguments **********/
    if(objc < 2){
	usage();
	return TCL_ERROR;
    }
    /********** Get Option **********/
    opt = Tcl_GetStringFromObj(objv[1], NULL);

    /*************** Read global variables ***************/
    TCL_GET(Double, paste, "left", left);
    TCL_GET(Double, paste, "right", right);
    TCL_GET(Double, paste, "top", start_time);
    TCL_GET(Double, paste, "bottom", end_time);
    TCL_GET(Double, gbl, "obs,lon", lon);
    TCL_GET(Double, gbl, "obs,lat", lat);
    TCL_GET(Int, gbl, "option,su,shotnumber",hw_shot);
    TCL_GET(Int, gbl, "option,su,sitenumber",hw_site);
    
    /************************************************************/
    /************************ Make cache ************************/
    /************************************************************/
    
    total_shot = make_cache_data(interp, start_time, end_time);

    /************************************************************/
    /************************ Operation *************************/
    /************************************************************/
    

    if (strcmp("makecache", opt) == 0) {
	return TCL_OK;
    } else if (strcmp("nearest", opt) == 0) {
	/***** Search nearest trace *****/
	double gx;

	void show_usage () {
	    Tcl_AppendResult(interp,
			     "wrong usage :\n",
			     "  getrh nearest gx\n",
			     "     gx       : graph x-coordinate\n",
			     (char *)NULL);
	};

	/** check arguments **/
	if( objc != 3 ||
	    TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[2], &gx) ) {
	    show_usage();
	    return TCL_ERROR;
	}
	i = -1;            /** ID of nearest trace **/
	min_dist = 1.0e30; /** init **/
	for(j=0;j<total_shot;j++){
	    /** read a record header info **/
	    if(min_dist > (dist=fabs(gbl_rh[j].x - gx))) {
		min_dist = dist;
		i = j;
	    }
	}
	if(i==-1){
	    Tcl_SetObjResult(interp, Tcl_NewIntObj(i));
	} else {
	    /** read a record header info **/
	    list = Tcl_NewListObj(0, NULL);
	    TCL_APPEND(list, Int, gbl_rh[i].hw[hw_shot]);
	    TCL_APPEND(list, Double, gbl_rh[i].x);
	    TCL_APPEND(list, Int, gbl_rh[i].hw[hw_site]);
	    TCL_APPEND(list, Int, gbl_rh[i].swdep);
	    Tcl_SetObjResult(interp, list);
	}
    } else if (strcmp("range", opt) == 0) {
	/***** Return rh-info in the display range *****/
	Tcl_Obj* item;

	list = Tcl_NewListObj(0, NULL);
	for(j=0;j<total_shot;j++){
	    if((right - gbl_rh[j].x)*(gbl_rh[j].x - left)>=0.0) {
		item = Tcl_NewListObj(0, NULL);
		TCL_APPEND(item, Int, gbl_rh[j].hw[hw_shot]);
		TCL_APPEND(item, Double, gbl_rh[j].x);
		TCL_APPEND(item, Int, gbl_rh[j].hw[hw_site]);
		Tcl_ListObjAppendElement(interp, list, item);
	    }
	}
	Tcl_SetObjResult(interp, list);
    } else if (strcmp("shotpos", opt) == 0) {
	/***** Return x-position(offset) of the specified shot number *****/
	int sn;
	
	void show_usage () {
	    Tcl_AppendResult(interp,
			     "wrong usage :\n",
			     "  getrh shotpos id\n",
			     "     id       : shot number\n",
			     (char *)NULL);
	};

	/** check arguments **/
	if(objc!=3 || TCL_ERROR == Tcl_GetIntFromObj(interp, objv[2], &sn) ) {
	    show_usage();
	    return TCL_ERROR;
	}

	for(j=0;j<total_shot;j++){
	    if( sn == gbl_rh[j].hw[hw_shot] ) {
		Tcl_SetObjResult(interp, Tcl_NewDoubleObj(gbl_rh[j].x));
		return TCL_OK;
	    }
	}
	Tcl_SetObjResult(interp, Tcl_NewDoubleObj(0.0));
	return TCL_ERROR;
    } else if (strcmp("minmax", opt) == 0) {
	/***** Return min and max offset *****/
	list = Tcl_NewListObj(0, NULL);
	TCL_APPEND(list, Double, gbl_rh[0].x);
	TCL_APPEND(list, Double, gbl_rh[total_shot-1].x);
	Tcl_SetObjResult(interp, list);
    } else {
	usage();
	return TCL_ERROR;
    }
    return TCL_OK;
}

