#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>

#include "pasteup.h"

/********************************************************************

 Get a single trace

********************************************************************/

int pasteup_gettraceProc(
			  ClientData clientData, /* not used                */
			  Tcl_Interp *interp,    /* current tcl interpreter */
			  int objc,              /* number of arguments     */
			  Tcl_Obj* CONST objv[]  /* arguments               */
			  )
{
    /********** Variables *********/
    /***** For tcl interaction *****/
    char *data_filename; /*** waveform filename ***/
    char *rh_filename;   /*** record header filename ***/
    int data_format;     /** waveform data format (SU, segy ...)**/
    int bfmt;            /** data binary format **/
    double start_time, end_time;
    double x;
    double agc_period;       /*** AGC(Auto Gain Control) period ***/
    int amp_correct;
    int usefilter, filter_type, causal_type;  /*** filter ***/
    int usecache;        /** flag : use cache or not **/
    double f1, f2, f3, f4;   /*** filter coef ***/
    int search_mode;         /*** trace search mode:
				  0 ... nearest
				  +1 ... right neighbor, +2 ... right neighbor2
				  -1 ... left neighbor, -2 ... left neighbor2 
				  9999 ... get maximum absolute amplitude
			     ***/
    int channel;           /** Channel number **/
    int hw_shot;           /** header word name for shot number         **/
    int hw_site;           /** header word name for site number         **/

    int n;
    float *a, *t, torg;
    /**** etc ****/
    FILE *fp;

    int i, j;

    double dist, min_dist;
    Tcl_Obj* ans;

    /********** Function **********/
    float gettrace_maxamp(int n, float *a, float *t,
			  float start_time, float end_time);
    void gettrace_draw_a_trace(Tcl_Interp *interp, int n, float *a, float *t,
			       float tmin, float tmax);
    
    /**********************************************/
    /******************** Main ********************/
    /**********************************************/
    
    /********** Get Option **********/
    Tcl_GetDoubleFromObj(interp, objv[1], &x);       /** x-position in km **/
    Tcl_GetIntFromObj(interp, objv[2], &search_mode);/** search mode **/

    /*************** Read global variables ***************/
    data_filename = TCL_GETSTRING(file_name, "wave");
    rh_filename = TCL_GETSTRING(file_name, "rh");
    TCL_GET(Int, gbl, "data,type", data_format);
    TCL_GET(Int, gbl, "data,byte", bfmt);
    TCL_GET(Int, gbl, "obs,channel", channel);

    TCL_GET(Double, wk, "trace,from", start_time);
    TCL_GET(Double, wk, "trace,to", end_time);
    TCL_GET(Int, gbl, "option,su,shotnumber", hw_shot);
    TCL_GET(Int, gbl, "option,su,sitenumber", hw_site);

    TCL_GET(Int, paste, "amp,correct", amp_correct);
    TCL_GET(Double, paste, "amp,agc", agc_period);

    TCL_GET(Int, gbl, "check,filter", usefilter);
    TCL_GET(Int, paste, "trace,usecache", usecache);
    TCL_GET(Int, filt, "offset", i);
    if(usefilter){
	TCL_GET(Int, filt, "type", filter_type);
	TCL_GET(Int, filt, "causal", causal_type);
	TCL_GET(Double, filt, "f1", f1);
	TCL_GET(Double, filt, "f2", f2);
	TCL_GET(Double, filt, "f3", f3);
	TCL_GET(Double, filt, "f4", f4);
	filter_type = (i==1) ? -filter_type : filter_type;
    } else {
	filter_type = (i==1) ? -100 : 0;
    }

    /********** read header info **********/
    if(total_shot < 1)
	total_shot = make_cache_data(interp, start_time, end_time);

    /************************************************************/
    /*********************** Preparations ***********************/
    /************************************************************/

    /*** memory allocation ***/
    if(NULL == (t = (float *)malloc(maxsample * sizeof(float))) ||
       NULL == (a = (float *)malloc(maxsample * sizeof(float))) ){
	fprintf(stderr, "Error: Memory allocation failed.\n");
	return TCL_ERROR;
    }

    /************************************************************/
    /********************* determine trace index ****************/
    /************************************************************/

    /*** Nearest trace ***/
    i = 0;                            /** ID of nearest trace**/
    min_dist = fabs(gbl_rh[0].x - x);
    for(j=i;j<total_shot;j++){
	if(min_dist > (dist=fabs(gbl_rh[j].x - x))) {
	    min_dist = dist;
	    i = j;
	}
    }
    if(search_mode != 9999) { /* 9999 is a flag for search maximum aplitude */
	i = i + search_mode;
    }
    if(i < 0||i>=total_shot) return(0);

    /***** trace info *****/
    ans = Tcl_NewListObj(0, NULL);
    TCL_APPEND(ans, Int, gbl_rh[i].hw[hw_site]);
    TCL_APPEND(ans, Int, gbl_rh[i].hw[hw_shot]);
    TCL_APPEND(ans, Double, gbl_rh[i].x);
    
    /************************************************************/
    /************************ Read trace ************************/
    /************************************************************/

    if (usecache) {
	/***** Read trace data from the cache *****/
	n = form_trace(gbl_a[i], gbl_t[i], gbl_n[i],
		       start_time, end_time, gbl_rh[i].sampling,
		       1.0, amp_correct, agc_period, 1.0, a, t);
    } else {
	/***** Read trace data from the file and correct amplitude *****/
	if(NULL==(fp=fopen(data_filename,"r"))){
	    fprintf(stderr, "Error: Cannot open waveform data file.\n");
	    return TCL_ERROR;
	}
	/** read : make a-array **/
	n = make_data(a, &torg, fp, data_format, bfmt, channel,
		      gbl_rh[i].start_point, gbl_rh[i].end_point, 
		      -gbl_rh[i].dsec, 10000.0,
		      gbl_rh[i].sampling, gbl_rh[i].dsec,
		      filter_type, causal_type, f1, f2, f3, f4);
	/** amplitude correction **/
	amplitude_correct(a, n, gbl_rh[i].sampling,
			  amp_correct, agc_period, 1.0, 1.0);
	/** make t-array **/
	for(j=0;j<n;j++){
	    t[j] = torg + (float)j/gbl_rh[i].sampling;
	}
	fclose(fp);
    }

    /************************************************************/
    /************************* draw trace ***********************/
    /************************************************************/

    if(search_mode == 9999) {
	/* get maximum amplitude */
	TCL_APPEND(ans, Double, gettrace_maxamp(n, a, t, start_time, end_time));
    } else {
	/* show trace */
	gettrace_draw_a_trace(interp, n, a, t, start_time, end_time);
    }
    
    /********** fin **********/
    Tcl_SetObjResult(interp, ans);
    return TCL_OK;
}

/********************************************************************
get maxamplitude
--------------------------------------------------------------------
n            : number of samples
a            : amp array
t            : time array
start_time  : start time to show
end_time    : end time to show
********************************************************************/
float gettrace_maxamp(int n, float *a, float *t,
		     float start_time, float end_time)
{
    float max_amp;
    int j;
    
    max_amp = 0.0;
    for(j=0;j<n;j++){
	if( (t[j] - start_time)*(end_time - t[j]) >= 0.0 ) {
	    if(fabs(a[j]) > max_amp) max_amp = fabs(a[j]);
	}
    }
    return(max_amp);
}

/********************************************************************
Draw a trace
--------------------------------------------------------------------
interp       : tcl interpreter
n            : number of samples
a            : amp array
t            : time array
start_time  : start time to show
end_time    : end time to show
********************************************************************/
void gettrace_draw_a_trace(Tcl_Interp *interp, int n, float *a, float *t,
			   float start_time, float end_time)
{
    /********** Local Variables *********/
    int j, k;
    char *canvas;        /*** trace canvas widget name ***/
    char com[256];
    Tcl_Obj* coordlist;
    int t_axis;
    int clip;            /** flag : clip or not **/
    int graph_width;
    float clip_value;
    double t_coef, amp, zero_level;

    /******************** Main *******************/
    canvas = TCL_GETSTRING(wk, "trace,widget,canvas");

    TCL_GET(Int, gbl, "t_axis", t_axis);
    TCL_GET(Double, wk, "trace,t_coef", t_coef);
    TCL_GET(Double, wk, "trace,amp", amp);
    TCL_GET(Double, wk, "trace,coord,x_zero", zero_level);
    TCL_GET(Int, wk, "trace,graph_width", graph_width);
    TCL_GET(Int, paste, "trace,clip", clip);

    /***** clear *****/
    sprintf(com, "%s delete all", canvas);
    Tcl_Eval(interp, com);
    
    /***** make waveform *****/
    coordlist = Tcl_NewListObj(0, NULL);
    Tcl_SetVar2Ex(interp, "coordlist", NULL, coordlist, 0);

    /** start time **/
    if (n<2) return;
    k = 0;
    if(start_time > t[0]) {
	j=1;
	while(start_time > t[j]) j++;
	a[k] = (a[j]*(start_time-t[j-1]) + a[j-1]*(t[j]-start_time))
	    / (t[j] - t[j-1]);
	t[k++] = start_time;
	    
    }

    /** main body **/
    for(j=1;j<n;j++) {
	if( (t[j] - start_time)*(end_time - t[j]) >= 0.0 ) {
	    a[k] = a[j];
	    t[k++] = t[j];
	}
    }
    
    /** end time **/
    if(end_time < t[n-1]) {
	j=0;while(end_time > t[j]) j++;
	a[k] = (a[j]*(end_time - t[j-1]) + a[j-1]*(t[j] - end_time))
	    / (t[j] - t[j-1]);
	t[k++] = end_time;
    }

    /** clip **/
    if(clip) {
	clip_value = graph_width / 2.0;
	for(j=0; j<k; j++) {
	    if( fabs(a[j]*amp) >= clip_value ) {
		a[j] = (a[j] <0) ? - clip_value/amp : clip_value/amp;
	    }
	}
    }

    /***** Convert coordinate into canvas coordinate *****/
    if (t_axis > 0) {
	/** downward axis **/
	double y_min;
	TCL_GET(Double, wk, "trace,coord,y_min", y_min);
	for(j=0; j<k; j++) {
	    TCL_APPEND(coordlist, Double, a[j]*amp + zero_level );
	    TCL_APPEND(coordlist, Double, y_min + (t[j] - start_time)*t_coef);
	}
    } else {
	/** upward axis **/
	double y_max;
	TCL_GET(Double, wk, "trace,coord,y_max", y_max);
	for(j=0; j<k; j++) {
	    TCL_APPEND(coordlist, Double, a[j]*amp + zero_level);
	    TCL_APPEND(coordlist, Double, y_max - (t[j] - start_time)*t_coef);
	}
    }

    /***** Draw *****/
    sprintf(com, "%s create line $coordlist -tag trace_graph -width 0.1", canvas);
    Tcl_Eval(interp, com);
}
