#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <tcl.h>

#include "pasteup.h"

/********************************************************************

Read wave-data and draw them in the vector format or Image format

********************************************************************/
int pasteup_getwaveProc(
			ClientData clientData, /* not used                */
			Tcl_Interp *interp,    /* current tcl interpreter */
			int objc,              /* number of arguments     */
			Tcl_Obj* CONST objv[]  /* arguments               */
			)
{
    /********** Variables *********/
    /***** Variables for tcl interaction *****/
    char *data_filename; /*** waveform filename ***/
    int bfmt;                /** data binary format (2:2byte int, etc)**/
    int mode;                /*** display style (wiggle, va...) ***/
    double amp;              /*** scale factor ***/
    int amp_step;            /*** threshold level ***/
    double tmin, tmax;
    double left, right;
    int wave_density;        /*** trace display freq ***/
    double wave_dist;    /*** trace display interval in km ***/
    double graph_width, graph_height;
    double left_offset, top_offset;
    int t_axis, amp_correct, amp_class, amp_cutoff;
    double geom_L, geom_a;   /** parameters for geometric spreading correct.**/
    double agc_period;       /*** AGC window (sec) ***/
    
    int image_mode;          /*** vector(0) ? image(1) ?  ***/
    int line_width;          /*** line width ***/
    int ncol;                /*** number of colors for VD ***/
    int canvas;              /*** canvas/graph flag ***/
    double dx;               /*** dx/dt for graph mode ***/
    int xaxis_direc;         /*** xaxis direction (1:normal, -1:reverse) ***/
    
    float clip;              /*** clip amplitude for GraphCanvas ***/
    float threshold;         /*** threshold level for VA ***/
    int fmt;                 /*** output format;
				  1:  vector on NormalCanvas
				  2:  images on NormalCanvas
				  -1: vector on GraphCanvas
				  -2: images on GraphCanvas
			     ***/
    double xrange, trange;

    /**** etc. ****/
    FILE *fp;
    
    int i, n;
    double x, kika_coef;
    double prev_x;
    float *t, *a, t_org;

    /*** For GD ***/
    gdImagePtr im;
    int color[BLACK+1];
    int image_format;      /*** image format, -1:jpeg, 1:png ***/
    int imagesize;
    char *image;

    /**********************************************/
    /******************** Main ********************/
    /**********************************************/
    
    /********** Get Option **********/
    Tcl_GetIntFromObj(interp, objv[1], &mode);      /** drawing mode **/
    Tcl_GetIntFromObj(interp, objv[2], &image_mode);/** image mode **/
    Tcl_GetDoubleFromObj(interp, objv[3], &amp);    /** scale factor **/

    /*************** Read global variables ***************/
    data_filename = TCL_GETSTRING(file_name, "wave");
    TCL_GET(Int, gbl, "data,byte", bfmt);
    TCL_GET(Double, paste, "left", left);
    TCL_GET(Double, paste, "right", right);
    TCL_GET(Double, paste, "top", tmin);
    TCL_GET(Double, paste, "bottom", tmax);
    TCL_GET(Double, paste, "graph_height", graph_height);
    TCL_GET(Double, paste, "graph_width", graph_width);
    TCL_GET(Double, paste, "left_margin", left_offset);
    TCL_GET(Double, paste, "top_margin", top_offset);
    TCL_GET(Double, paste, "wave_distance", wave_dist);
    TCL_GET(Int, gbl, "t_axis", t_axis);
    TCL_GET(Int, paste, "wave_density", wave_density);

    TCL_GET(Int, paste, "amp,threshold", amp_step);
    TCL_GET(Int, paste, "amp,correct", amp_correct);
    TCL_GET(Int, paste, "amp,cutoff", amp_cutoff);
    TCL_GET(Int, paste, "amp,class", amp_class);
    TCL_GET(Double, paste, "amp,agc", agc_period);

    TCL_GET(Double, paste, "amp,geometric,standard", geom_L);
    TCL_GET(Double, paste, "amp,geometric,power", geom_a);
    TCL_GET(Int, paste, "ncolors", ncol);

    TCL_GET(Int, gbl, "data,xaxis_direction", xaxis_direc);
    
    TCL_GET(Int, gbl, "line_width", line_width);
    if(image_mode!=0){
	line_width = line_width + line_width%2 - 1;
    }
    TCL_GET(Int, paste, "image,format", image_format);
    TCL_GET(Int, paste, "canvas,mode", canvas);

    /************************************************************/
    /************************ Make cache ************************/
    /************************************************************/
    
    total_shot = make_cache_data(interp, tmin, tmax);

    /************************************************************/
    /*********************** Preparations ***********************/
    /************************************************************/

    /*** amplification ***/
    amp *= compute_amp_coefficient(mode);
    amp_step = compute_amp_step(bfmt, mode, amp_step);

    /*** memory allocation ***/
    if(NULL == (t = (float *)malloc(maxsample * sizeof(float))) ||
       NULL == (a = (float *)malloc(maxsample * sizeof(float))) ){
	fprintf(stderr, "Error: Memory allocation failed.\n");
	return TCL_ERROR;
    }

    /*** determine output format ***/
    fmt = (image_mode==1) ? 2 : 1;
    fmt = (canvas == 1) ? fmt : -fmt;
    /*** graph_width/graph_height, left_offset/right_offset ***/
    switch(fmt) {
	case 1: /*** Vector on NormalCanvas ***/
	    xrange = graph_width;
	    trange = graph_height;
	    declare_color_codes(interp, color, mode, ncol);
	    im = gdImageCreateTrueColor(xrange+1,trange+1);
	    break;
	case 2: /*** Image on NormalCanvas ***/
	    xrange = graph_width;
	    trange = graph_height;
	    left_offset = 0;
	    top_offset = 0;
	    im = gdImageCreateTrueColor(xrange+1, trange+1);
	    declare_colors(interp, im, color, mode, ncol);
	    gdImageFilledRectangle(im, 0, 0, xrange+1, trange+1, color[WHITE]);
	    break;
	case -1: /*** Vector on GraphCanvas ***/
	    xrange = right - left;
	    trange = tmax - tmin;
	    left_offset = left;
	    top_offset = tmin;
	    im = gdImageCreateTrueColor(xrange+1,trange+1);
	    declare_color_codes(interp, color, mode, ncol);
	    amp *= xrange / graph_width;
	    clip = (amp_cutoff < 1) ? -1.0 : xrange*amp_cutoff/graph_width;
	    threshold = (amp_step < 1) ? 0.0 : xrange*amp_step/graph_width;
	    break;
	default:
	case -2: /*** Image on GraphCanvas ***/
	    TCL_GET(Double, paste, "graph,dx",dx);
	    xrange = (right - left) / dx;
	    while(xrange > 4500) { xrange /= 2.0; }
	    xrange = (graph_width > xrange) ? graph_width : xrange;

	    TCL_GET(Double, paste, "graph,dt",dx);
	    trange = (tmax - tmin) / dx;
	    while(trange > 4500) { trange /= 2.0; }
	    trange = (graph_height > trange) ? graph_height : trange;
	    left_offset = 0;
	    top_offset = 0;
	    im = gdImageCreateTrueColor(xrange + 1, trange + 1);
	    declare_colors(interp, im, color, mode, ncol);
	    gdImageFilledRectangle(im, 0, 0, xrange+1, trange+1, color[WHITE]);
	    break;
    }

    /************************************************************/
    /***************** Draw by using cache data *****************/
    /************************************************************/

    prev_x = gbl_rh[0].x - wave_dist*2.0;
    for(i=0;i<total_shot;i+=wave_density){
	/*** check plot or not ***/
	x = gbl_rh[i].x;
	if( (x - left)*(right - x) < 0.0 ||
	    fabs(prev_x - x) < wave_dist    ) continue;
	prev_x = x;
	kika_coef = pow(fabs(x)/geom_L, geom_a);

	/*** form trace from cache ***/
	n = form_trace(gbl_a[i], gbl_t[i], gbl_n[i],
		       tmin, tmax, gbl_rh[i].sampling,
		       amp, amp_correct, agc_period, kika_coef, a, t);
	    
	/*** plot ***/
	if(fmt == -1) {
	    draw_graph(interp, mode, n, a, t, x, clip, threshold);
	} else {
	    if(xaxis_direc==0) {
		x = (x - left) / (right - left) * xrange + left_offset;
	    } else {
		x = (right - x) / (right - left) * xrange + left_offset;
	    }
	    if(fmt == 1) {
		draw_vector(interp, a, t, n, mode, t_axis,
			    amp_step, amp_cutoff,
			    tmin, tmax, x, trange, top_offset, color);
	    } else {
		draw_image(im, n, a, t, mode, t_axis,
			   amp_step, amp_cutoff,
			   tmin, tmax, gbl_rh[i].sampling,
			   x, trange, top_offset, line_width, color);
	    }
	}
    }

    if(image_mode==1){
	/********** Make PNG/JPEG file **********/
	if(image_format > 0) image = (char *)gdImagePngPtr(im, &imagesize);
	else            image = (char *)gdImageJpegPtr(im, &imagesize, 90);
	Tcl_SetObjResult(interp, Tcl_NewByteArrayObj(image, imagesize));
	gdFree(image);
    }
    gdImageDestroy(im);

    free(t);
    free(a);
    return TCL_OK;
}

