#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <tcl.h>
#include <sys/stat.h>

#include "pasteup.h"

int make_cache_data(
		   Tcl_Interp *interp,    /* current tcl interpreter */
		   double start_time, double end_time
		   )
{
    /********** Local Variables **********/
    int condition;    /* cache conditon                       */
    double velocity_coef;
    float coef_rv;
    char script[128];
    FILE *fp;
    int i;

    char *data_filename;/* waveform filename                  */
    char *rh_filename;/* record header filename               */
    double lon, lat;  /* OBS lon and lat (for THK data)       */
    int dist_axis;    /* Line direction (East, North)         */
    int channel;      /* Channel number                       */
    int data_format;  /* waveform data format (SU, SEG-Y...)  */
    int data_byte;    /* size of one binary sample            */
    int correct_shot; /* Flag for shot time correction        */
    double obs_depth; /* OBS depth (m)                        */
    double water_velocity;/* Water wave vel (km/sec)          */
    double sediment_velocity;/* Sediment vel (km/sec)         */
    double rd_vel;    /* reduction velocity (km/sec)          */
    int usefilter;    /* flag for time domain filter          */
    int filter_type;  /* filter type                          */
    int causal_type;  /* filter causality                     */
    double f1, f2, f3, f4;/* Filter coefficients              */
    int stack;        /* hyperbola(1), slant(2) or not(0).    */
    double stack_range;/* stacking range in Km                */
    double stack_vel; /* stacking Velocigy in Km/sec.         */
    double stack_time;/* stacking time                        */
    double cache_len; /* cache read length in second          */

    /************************************************************/
    /***************** Read prameters from Tclsh ****************/
    /************************************************************/

    /***** filenames *****/
    data_filename = TCL_GETSTRING(file_name, "wave");
    rh_filename = TCL_GETSTRING(file_name, "rh");
    /***** obs position *****/
    TCL_GET(Double, gbl, "obs,lon", lon);
    TCL_GET(Double, gbl, "obs,lat", lat);
    /***** data format *****/
    TCL_GET(Int, gbl, "data,type", data_format);
    TCL_GET(Int, gbl, "data,byte", data_byte);
    /***** OBS position related, channels *****/
    if(data_format == 3) {
	TCL_GET(Int, gbl, "obs,direc", dist_axis);
    } else {
	TCL_GET(Int, gbl, "data,xaxis", dist_axis);
    }
    TCL_GET(Int, gbl, "obs,channel", channel);
    /***** depths *****/
    TCL_GET(Int, paste, "correct,shot_mode", correct_shot);
    TCL_GET(Double, gbl, "obs,depth", obs_depth);
    /***** velocities *****/
    TCL_GET(Double, gbl, "water_v", water_velocity);
    TCL_GET(Double, gbl, "sediment_v", sediment_velocity);
    TCL_GET(Double, paste, "reduction", rd_vel);
    /***** stack *****/
    TCL_GET(Int, paste, "stack", stack);
    TCL_GET(Double, paste, "stack,range", stack_range);
    TCL_GET(Double, paste, "stack,vel", stack_vel);
    TCL_GET(Double, paste, "stack,time", stack_time);
    /***** filter *****/
    TCL_GET(Int, gbl, "check,filter", usefilter);
    TCL_GET(Int, filt, "offset", i);
    if(usefilter){
	TCL_GET(Int, filt, "type", filter_type);
	TCL_GET(Int, filt, "causal", causal_type);
	TCL_GET(Double, filt, "f1", f1);
	TCL_GET(Double, filt, "f2", f2);
	TCL_GET(Double, filt, "f3", f3);
	TCL_GET(Double, filt, "f4", f4);
	filter_type = (i==1) ? -filter_type : filter_type;
    } else {
	filter_type = (i==1) ? -100 : 0;
    }

    /***** cache : cache has 1.0 sec margins *****/
    TCL_GET(Double, gbl, "option,cache_length", cache_len);
    start_time = start_time - cache_len - 1.0;
    end_time = end_time + cache_len + 1.0;

    /***** Check input file existence *****/
    {
	struct stat buf;
	if(stat(data_filename, &buf) == -1) {
	    return(-1);
	}
    }

    /************************************************************/
    /*********************** Record Header **********************/
    /************************************************************/

    TCL_GET(Int, wk, "condition", condition);

    if(condition < -10 || condition == 0) {
	/***** Read header info *****/
	total_shot = read_header(data_format, data_byte,
				 rh_filename, data_filename,
				 lon, lat, dist_axis, channel);
	sprintf(script, "set wk(total_shot) %d", total_shot);
	Tcl_Eval(interp, script);
	/***** sort traces(record header) *****/
 	sort_record_header(total_shot);

	/***** shot time correction *****/
	switch(correct_shot) {
	    default: /* none */
		velocity_coef = 0.0;
		water_velocity = 0.0;
		break;
	    case 2: /* shots move to sea floor */
		velocity_coef = 0.0;
		water_velocity = 1.0/(water_velocity * 1000.0);
		break;
	    case 1: /* flat sea floor correction */
		obs_depth *= 1000.0;  /** convert km into meter **/
		velocity_coef = (1.0/sediment_velocity - 1.0/water_velocity)/1000.0;
		water_velocity = 0.0;
		break;
	}
	/***** reduction *****/
	coef_rv = (rd_vel != 0.0) ? 1.0/rd_vel : 0.0;

	/***** Apply dt *****/
	for(i=0;i<total_shot;i++){
	    /** shot time correction **/
	    gbl_rh[i].shift =
		+ fabs(gbl_rh[i].offset)*coef_rv
		+ (obs_depth-gbl_rh[i].swdep)*velocity_coef
		+ (float)gbl_rh[i].swdep * water_velocity;
	    /** clock correction **/
	    gbl_rh[i].dsec += gbl_rh[i].shift;
	}
    }

    /************************************************************/
    /********************** Make cache data *********************/
    /************************************************************/

    if(condition < 0 && total_shot < maxtrace) {
	/***** file existence check *****/
	if(NULL==(fp=fopen(data_filename,"r"))){
	    fprintf(stderr, "Error: Cannot open waveform data file.\n");
	    return TCL_ERROR;
	}

	for(i=0;i<total_shot;i++){
	    gbl_n[i] = make_one_trace(i, fp, data_format, data_byte, channel,
				      start_time, end_time, rd_vel,
				      filter_type, causal_type, f1, f2, f3, f4,
				      stack,
				      stack_range, stack_vel, stack_time,
				      gbl_a[i], &gbl_t[i]);
	}
	fclose(fp);
	/***** update cache data info : cache has 1.0sec margin *****/
	sprintf(script, "set wk(prev,tmin) %g", start_time+1.0);
	Tcl_Eval(interp, script);
	sprintf(script, "set wk(prev,tmax) %g", end_time-1.0);
	Tcl_Eval(interp, script);
    }

    /***** update condition *****/
    sprintf(script, "set wk(condition) 1");
    Tcl_Eval(interp, script);

    return(total_shot);
}
