#!/usr/local/bin/ruby

# --------------------------------------------------------------------
# Generate record header file using in Pasteup, for CHIBA waveform data.
# --------------------------------------------------------------------
# Descriptions:
# This script needs three intpu files:
#    1. shot info (CHIBA format)
#    2. record header (CHIBA format)
#    3. parameter file for xplottd program (CHIBA program)
# Output file format is,
# <shot_no> <day> <hour> <min> <sec> <msec> <dist(km)> <samp(Hz)> \
#                      <sblock> <nblock> <rblock>
#
# offset distance is calculated using Bessel ellipse.
# --------------------------------------------------------------------

# --------------------------------------------------------------------
# Package and Constants  ---------------------------------------------
# --------------------------------------------------------------------
include Math

# --------------------------------------------------------------------
# Defun --------------------------------------------------------------
# --------------------------------------------------------------------
def read_param(param)
  open(param) do |f|
    # Neglect first 5 lines.
    for i in 1..5
      f.gets 
    end
    # get lat and lon
    dat = f.gets.split
    lat = dat[0].to_f * PI / 180.0
    lon = dat[1].to_f * PI / 180.0
    # Neglect more 15 lines
    for i in 1..15
      f.gets 
    end
    dat = f.gets.split
    direc = dat[1].to_i
    [lat, lon, direc]
  end
end

def asin(s)
  c = 1 - s * s
  atan2(s, c)
end

def calc_dist(lat, lon, obs_lat, obs_lon, direc)
  if direc == 2 then
    # north - south mode
    fact = lat >= obs_lat ? 1.0 : -1.0
  else
    # east - west mode
    fact = lon >= obs_lon ? 1.0 : -1.0
  end
  coef = 11.55/60.0 * PI / 180.0
  lat    = lat - coef * sin(2.0 * lat)
  obs_lat = obs_lat - coef * sin(2.0 * obs_lat)
  ae = cos(lat)*cos(lon)
  be = cos(lat)*sin(lon)
  ce = sin(lat)
  as = cos(obs_lat)*cos(obs_lon)
  bs = cos(obs_lat)*sin(obs_lon)
  cs = sin(obs_lat)
  mid = sqrt((ae-as)*(ae-as) + (be-bs)*(be-bs) + (ce-cs)*(ce-cs));
  delta = 4.0 * asin(mid/4.0);
  dist = 6369.0 * delta * fact;
end

def search_shot(header, n)
  header.each do |line|
    dat = line.split
    if dat[0].to_i == n then
      return line
    end
  end
  STDERR.puts "Cannot find #{n} shots in original header file."
  return ""
end

# --------------------------------------------------------------------
# Main ---------------------------------------------------------------
# --------------------------------------------------------------------

# --------------------------------------------------------------------
# Check arguments ----------------------------------------------------
if ARGV.size != 3 then
  STDERR.puts "Usage:\n" +
    "#{$0} <header> <shot> <param>\n" +
    "\t<header>  :  Original header file.\n" +
    "\t<shot>    :  Shot information file.\n" +
    "\t<param>   :  Parameter file for xplottd.\n"
  exit
end
# get file name
header = ARGV.shift
shot   = ARGV.shift
param  = ARGV.shift

# --------------------------------------------------------------------
# Read parameter file and get OBS postion and line direction.  -------
obs_lat, obs_lon, direc = read_param(param)

# --------------------------------------------------------------------
# Read original header file and shot info file -----------------------
h = open(header) do |f|
  f.readlines
end
s = open(shot) do |f|
  f.readlines
end

# --------------------------------------------------------------------
# Make data ----------------------------------------------------------
d = []
s.each do |line|
  # basic info from shot file
  dat = line.split
  shot_no = dat[0].to_i
  month   = dat[1].to_i
  day     = dat[2].to_i
  hour    = dat[3].to_i
  min     = dat[4].to_i
  sec     = dat[5].to_f
  lat     = dat[6].to_f * PI / 180.0
  lon     = dat[7].to_f * PI / 180.0
  dist    = calc_dist(lat, lon, obs_lat, obs_lon, direc)
  # additional info from orig-header
  dat = search_shot(h, shot_no)
  if dat == "" then
    next
  end
  samp    = dat[16,8].to_f
  sblock  = dat[30,5].to_i
  nblock  = dat[35,5].to_i
  rblock  = dat[40,5].to_i
  printf("%5d %2d %2d %2d %2d %8.3f %8.3f %8.3f %5d %5d %5d\n",
	 shot_no, month, day, hour, min, sec, dist,
	 samp, sblock, nblock, rblock)
end

