/********************************************************************
Header file for C programs of PasteUP
********************************************************************/

#ifndef _PASTEUP_H
#define _PASTEUP_H 1

#include <tcl.h>
#include "record_header.h"

/* Parameters length from Tcl(wish) to C  */
#define MAX_STDIN_LENGTH 1200

/* General */
#define ON  1
#define OFF 0

/* Filter type */
#define CAUSAL 1
#define UNCAUSAL 2

#define NOFILTER 100
#define LOWPASS  1  /** Chebyshev, low pass **/
#define HIGHPASS 2  /** Chebyshev, high pass **/
#define BANDPASS 3  /** Chebyshev, band pass **/
#define BANDSTOP 4  /** Chebyshev, band stop **/
#define BUTBPASS 5  /** Butterworth, band pass **/
#define BUTLPASS 6  /** Butterworth, low pass **/
#define BUTHPASS 7  /** Butterworth, high pass **/
#define OFFSETFILTER -1

/* orientation */
#define EAST  1
#define NORTH 2

/* x-axis */
#define OFFSET 0
#define TRACL  1
#define CDP    2
#define SX     3
#define SY     4
#define GX     5
#define GY     6
#define FLDR   7
#define TRACF  8

/* Color assignment */
#define NCOLOR 256  /** Number of colors for red, blue, gray, respectively **/
#define NRAINBOW 1280
#define TRANSPARENT -1
#define RED   32768
#define BLUE  32769
#define WHITE 32770
#define BLACK 32771

/* Drawing style : more than 10 means VD style */
#define WIGGLE_MODE       1
#define VA_MODE           2
#define VA_REDBLUE_MODE   3
#define VA_WITHOUT_WIGGLE 4
#define VA_WO_WIGGLE_RB   5

/* Waveform data format */
#define KASA_FORMAT  1         /** Kasahara-Lab format **/
#define SU_FORMAT    2         /** Seismic Unix format **/
#define THK_FORMAT   3         /** TOHOKU Univ. format **/
#define CHIBA_FORMAT 4         /** CHIBA, ORI   format **/
#define SEGY_FORMAT  5         /** SEG-Y,IASPEI format **/
#define SEGY1_FORMAT 6         /** SEG-Y,1Ch IASPEI format **/

/* SU/SEG-Y Headers */
#define RECORD_HEADER_TRACL 0 /** TRACL **/
#define RECORD_HEADER_TRACR 1 /** TRACR **/
#define RECORD_HEADER_FLDR  2 /** FLDR  **/
#define RECORD_HEADER_TRACF 3 /** TRACF **/
#define RECORD_HEADER_EP    4 /** EP    **/

/* binary data sample format */
#define FMT_2BYTE_INT        2 /** 2byte integer      **/
#define FMT_3BYTE_INT        3 /** 3byte integer      **/
#define FMT_3BYTE_INT_SW    -3 /** 3byte integer,swap **/
#define FMT_4BYTE_INT        4 /** 4byte integer      **/
#define FMT_4BYTE_INT_SW    -4 /** 4byte integer,swap **/
#define FMT_4BYTE_FLOAT     14 /** float              **/
#define FMT_4BYTE_FLOAT_SW -14 /** float, swap        **/
#define FMT_4BYTE_IBM       15 /** IBM float          **/
#define FMT_4BYTE_IBM_SW   -15 /** IBM float, swap    **/

/* encodeing */
#define ENCODE_STRING "drawing record section"

/********************************************************************
Global Variable(s)
********************************************************************/

#ifndef _PASTEUP_MAIN_
extern int total_shot;
extern struct record_header *gbl_rh;
extern float **gbl_a;
extern float *gbl_t;
extern int *gbl_n;
extern int maxtrace;
extern int maxsample;
#endif

/********************************************************************
Include files
********************************************************************/
#include <gd.h>

/********************************************************************
Macros
********************************************************************/
#define TCL_GET(a,b,c,d)\
 Tcl_Get ## a ## FromObj(interp, \
                   Tcl_GetVar2Ex(interp, #b, c, TCL_GLOBAL_ONLY), &d)
#define TCL_GETSTRING(a, b)\
  Tcl_GetStringFromObj(Tcl_GetVar2Ex(interp, #a, b, TCL_GLOBAL_ONLY), NULL)
#define TCL_APPEND(a,b,c)\
 Tcl_ListObjAppendElement(interp, a, Tcl_New ## b ## Obj( c ))
#define PASTEUP_ERROR fprintf(stderr, "\tError : line %d in file %s.\n", \
			      __LINE__, __FILE__)

#define KOKO fprintf(stderr, "\tDebug : line %d in file %s.\n",\
                     __LINE__, __FILE__)

/********************************************************************
Functions in modules.
********************************************************************/

/***** libmalloc.c *****/
extern float **fmalloc2(int nx, int nz);
extern void ffree2(float **p);
extern int realloc_global_variable();

/***** libmakecache.c *****/
int make_cache_data(Tcl_Interp *interp, double start_time, double end_time);

/***** sort_rh.c *****/
void sort_record_header(int n);

/***** make_color.c *****/
void declare_color_codes(Tcl_Interp *interp, int *color, int mode, int ncol);
void declare_colors(Tcl_Interp *interp,
		    gdImagePtr im, int *color, int mode, int ncol);
int amp2index(float amp, int step);

/***** read_header.c *****/
int read_header(int data_format, int data_byte, char *rh, char *wave,
		double lon, double lat, int dist_axis, int channel);

/***** make_one_trace.c *****/
int make_one_trace(int i, FILE *fp, int data_format, int data_type, int ch,
		   float start_time, float end_time, double rd_vel,
		   int filter_type, int causal_type, 
		   double f1, double f2, double f3, double f4,
		   int stack, double stack_range,
		   double stack_vel, double stack_time,
		   float *a, float *t);
int form_trace(float *a_org, float t_org, int n_org, float st, float et,
	       float sampling, double amp, int amp_correct, double agc,
	       double kika_coef, float *a, float *t);

/***** amp_correct.c *****/
void amplitude_correct(float *a, int count, double sampling,
		       int amp_correct, double agc_period,
		       double scale, double kika_coef);
float compute_amp_coefficient(int mode);
int compute_amp_step(int data_type, int mode, int amp_step);
void amp_clip(float *a, int count, int clip);
void amp_clip_float(float *a, int count, float clip);

/***** make_data.c *****/
int make_data(float *a, float *t,
	      FILE *fp, int data_format, int data_byte, int ch,
	      int data_point1, int data_point2,
	      float start_time, float end_time,
	      float sampling, float dt_clock,
	      int filter_type, int causal_type,
	      double f1, double f2, double f3, double f4);

/***** stack.c *****/
int make_stacked(int i, int data_format, int data_byte, int ch,
		 float *a, float *t_out, float start_time, float end_time,
		 float rd_vel, int stack,
		 double stack_range, double stack_vel, double stack_time,
		 int filter_type, int causal,
		 double f1, double f2, double f3, double f4, FILE *fp);

/***** read_data.c *****/
int read_data(float *a, float *t,
	      FILE *fp, int data_format, int data_byte, int ch,
	      int data_point1, int data_point2,
	      float start_time, float end_time,
	      float sampling, float dt_clock);

/***** filter.c *****/
void filter(float *a, int count, double sampling,
	    int filter_type, int causal_type,
	    double f1, double f2, double f3, double f4);

/***** va.c *****/
int search_threshold(int n, float *ag, float *tg, float *va, float level);
int search_threshold_minus(int n, float *ag, float *tg, float *va,float level);
float interpolate_amp(int n, float t, float *ag, float *tg);

/***** file.c *****/
FILE *open_file(char *filename, char *file_type);
FILE *open_write_file(char *filename, char *file_type);

/***** draw_vector_trace.c *****/
void draw_vector(Tcl_Interp *interp,
		 float *a, float *t, int count, int mode, int t_axis,
		 int amp_step, int amp_cutoff, float tmin, float tmax,
		 double x, double height, double top_offset, int *color);

/***** draw_image_trace.c *****/
void draw_image(gdImagePtr im, int n, float *a, float *t, int mode,
		int t_axis, int amp_step, int amp_cutoff,
		float tmin, float tmax, float sampling, double x,
		double height, double top_offset, int lw, int color[]);
/***** draw_graph_trace.c *****/
void draw_graph(Tcl_Interp *interp, int mode, int n, float *a, float *t,
		double x, float clip, float threshold);
/***** su.c *****/
extern float su_dist_conv(float org_x, float scale, int counit);

#endif /* !_PASTEUP_H */
