#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "segy.h"
#include "endian.h"
#include "pasteup.h"

static char swap;

/*
  --------------------------------------------------------------------
*/

static short int read_short_int(FILE *fp, int offset)
/********************************************************************
 Read one short int data from file "fp", position "offset".
 ********************************************************************/
{
     /***** Local Variables *****/
     short int tmp;

     /********** Main **********/
     fseek(fp, offset, SEEK_SET);
     fread(&tmp, sizeof(tmp), 1, fp);
     if(swap)SWAP_SI(tmp);
     return(tmp);
}

/*
  --------------------------------------------------------------------
*/

/***********************************************************************
 Convert IBM float into native float (one value)
*************************************************************************/
static void ibm_to_float(int *a)
{
    /***** Local Variables *****/
    int tmp;
    int mask, t;

    /********** Main **********/
    tmp = *a;
    mask = 0x00ffffff & tmp;
    if (!tmp || (mask == 0) ){*a = 0;	return;}
    t = (int) ((0x7f000000 & tmp) >> 22) - 130;
    while (!(mask & 0x00800000)) { --t; mask <<= 1; }
    if (t > 254)
	tmp = (0x80000000 & tmp) | 0x7f7fffff;
    else if (t <= 0)
	tmp = 0;
    else
	tmp = (0x80000000 & tmp) |(t << 23)|(0x007fffff & mask);
    *a = tmp;
}

/*
  --------------------------------------------------------------------
*/

/********************************************************************
 Read a 4-byte interger data from file "fp".
 ********************************************************************/
static int get_4byte_int(FILE *fp)
{
    /***** Local Variables *****/
    int tmp;
    
    /********** Main **********/
    fread(&tmp, sizeof(tmp), 1, fp);
    if(swap)SWAP_4I(tmp);
    return(tmp);
}

/*
  --------------------------------------------------------------------
*/

/********************************************************************
 Read one short int data from file "fp".
 ********************************************************************/
static short int get_short_int(FILE *fp)
{
     /***** Local Variables *****/
     short int tmp;

     /********** Main **********/
     fread(&tmp, sizeof(tmp), 1, fp);
     if(swap)SWAP_SI(tmp);
     return(tmp);
}

/*
  --------------------------------------------------------------------
*/

/********************************************************************
 Estimate number of trace in a file 'fn'
 --------------------------------------------------------------------
 [argumnets]
 fn    : "SEGY" file name
 byte  : data size (2 or 4 byte)
 n     : number of samples in a trace.
 ********************************************************************/
int segy_compute_trace_number(char *fn, int byte, int n)
{
    /***** Local Variables *****/
    struct stat fstat;
     
    /********** Main **********/
    stat(fn, &fstat);
    return( (fstat.st_size - REEL_HEADER)/(TRACE_HEADER_LENGTH + byte * n) );
}

/*
  --------------------------------------------------------------------
*/

/********************************************************************
 Identify data byte
 ********************************************************************/
int segy_data_byte(short int data_format)
{
    switch(data_format) {
	case 1:
	case 2:
	case 4:
	    return(4);
	    break;
	case 3:
	    return(2);
	    break;
	default:
	    fprintf(stderr, "ERROR: segy_data_type: Invalid data type %d\n",
		    data_format);
	    break;
    }
    return(4);  /** default **/
}

/*
  --------------------------------------------------------------------
*/

/********************************************************************
 Read SEG-Y Reel Identification Header.
 --------------------------------------------------------------------
 [Arguments]
 fp  : File pointer for SEG-Y file for input.
 ********************************************************************/
struct segy_reel_header read_segy_reel_header(FILE *fp, char swap_mode)
{
     /***** Local Variables *****/
     struct segy_reel_header h;

     /********** Main **********/
     swap = swap_mode;

     /***** Number of data traces per record *****/
     h.n_trace_gather = read_short_int(fp, REEL_HEADER_PART1 + SEGY_TRACE_NO);

     /***** Sampling interval *****/
     h.sampling = read_short_int(fp, REEL_HEADER_PART1 + SEGY_SAMPLE_INTERVAL);

     /***** SEG-Y Format Version *****/
     h.version = read_short_int(fp, REEL_HEADER_PART1 + SEGY_VERSION);

     /***** Number of samples per trace *****/
     h.n_sample = read_short_int(fp, REEL_HEADER_PART1 + SEGY_TRACE_SAMPLES);

     /***** Data sample format code *****/
     h.data_format = read_short_int(fp, REEL_HEADER_PART1 + SEGY_DATA_FORMAT);

     /***** *****/

     return(h);
}

/*
  --------------------------------------------------------------------
*/

/********************************************************************
Read record header information from SEGY
Return total number of traces.
********************************************************************/
int segy_read_rh(char *fn, int bfmt, int xaxis)
{
    /********** Local Variables **********/
    FILE *fp;
    int i, n, data_byte;
    struct segy_reel_header header;
    struct segy_iaspei_header tr;
    struct tm tt;

    /**********************************************/
    /******************** Main ********************/
    /**********************************************/
    
    swap = (bfmt < 0) ? SWAP_ON : SWAP_OFF;
    data_byte = (abs(bfmt) > 10) ? 4 : abs(bfmt);

    /********** Read SEG-Y Reel header information **********/
    fp = open_file(fn, "waveform (SEGY, IASPEI, rh)");
    header = read_segy_reel_header(fp, swap);
    header.n_trace = segy_compute_trace_number
	(fn, segy_data_byte(header.data_format), header.n_sample);
    
    n = 0;
    fseek(fp, 3600, SEEK_SET);
    for(i=0;i<header.n_trace;i++){
	/***** Read header *****/
	fread(&tr, sizeof(struct segy_iaspei_header), 1, fp);
	if(swap) {
	    SWAP_4I(tr.tracl);
	    SWAP_4I(tr.tracr);
	    SWAP_4I(tr.fldr);
	    SWAP_4I(tr.tracf);
	    SWAP_4I(tr.ep);
	    SWAP_4I(tr.offset);
	    SWAP_4I(tr.sdepth);
	    SWAP_4I(tr.swdep);
	    SWAP_4I(tr.gwdep);

	    SWAP_SI(tr.counit);
	    SWAP_SI(tr.scalco);
	    SWAP_4I(tr.sx);
	    SWAP_4I(tr.sy);
	    SWAP_4I(tr.gx);
	    SWAP_4I(tr.gy);
	    SWAP_SI(tr.tstat);
	    SWAP_SI(tr.delrt);
	    SWAP_SI(tr.laga);
	    SWAP_SI(tr.lagb);
	    SWAP_SI(tr.ns);
	    SWAP_SI(tr.dt);
	    SWAP_SI(tr.year);
	    SWAP_SI(tr.day);
	    SWAP_SI(tr.hour);
	    SWAP_SI(tr.minute);
	    SWAP_SI(tr.sec);
  	}
	gbl_rh[n].start_point = ftell(fp);
	gbl_rh[n].end_point   = tr.ns;

	gbl_rh[n].hw[RECORD_HEADER_TRACL] = tr.tracl;
	gbl_rh[n].hw[RECORD_HEADER_TRACR] = tr.tracr;
	gbl_rh[n].hw[RECORD_HEADER_FLDR]  = tr.fldr;
	gbl_rh[n].hw[RECORD_HEADER_TRACF] = tr.tracf;
	gbl_rh[n].hw[RECORD_HEADER_EP]    = tr.ep;

	gbl_rh[n].sdepth      = tr.sdepth;
	gbl_rh[n].swdep       = tr.swdep;
	gbl_rh[n].gwdep       = tr.gwdep;
	gbl_rh[n].sampling    = 1.0e6/(float)(tr.dt);

	tt.tm_year = tr.year - 1900;
	tt.tm_mon  = 0;
	tt.tm_mday = tr.day;
	tt.tm_hour = tr.hour;
	tt.tm_min  = tr.minute;
	tt.tm_sec  = tr.sec;
	gbl_rh[n].st = mktime(&tt);
	gbl_rh[n].msec = 0.0;

	gbl_rh[n].counit      = tr.counit;
	gbl_rh[n].scalco      = tr.scalco;
	gbl_rh[n].sx          = tr.sx;
	gbl_rh[n].sy          = tr.sy;
	gbl_rh[n].gx          = tr.gx;
	gbl_rh[n].gy          = tr.gy;

	gbl_rh[n].dsec        =
	    -(float)(tr.delrt + tr.lagb - tr.laga)/1000.0;
	gbl_rh[n].offset      = tr.offset / 1000.0;

	/***** set offset (offset/tracl/cdp) *****/
	switch(xaxis) {
	    case OFFSET: /** offset **/
		gbl_rh[n].x    = gbl_rh[n].offset;
		break;
	    case FLDR:
		gbl_rh[n].x = tr.fldr;
		break;
	    case TRACF:
		gbl_rh[n].x = tr.tracf;
		break;
	    case SX:     /** sx     **/
		gbl_rh[n].x = su_dist_conv(tr.sx, tr.scalco, tr.counit);
		break;
	    case SY:     /** sy     **/
		gbl_rh[n].x = su_dist_conv(tr.sy, tr.scalco, tr.counit);
		break;
	    case GX:     /** gx     **/
		gbl_rh[n].x = su_dist_conv(tr.gx, tr.scalco, tr.counit);
		break;
	    case GY:     /** gy     **/
		gbl_rh[n].x = su_dist_conv(tr.gy, tr.scalco, tr.counit);
		break;
	    case TRACL:  /** tracl  **/
		gbl_rh[n].x = tr.tracl;
		break;
	    case CDP:    /** cdp    **/
		gbl_rh[n].x = tr.cdp;
		break;
	}
	/** seek to next trace **/
	fseek(fp, data_byte*gbl_rh[n].end_point, SEEK_CUR);
	n++;

	/***** require re-allocation ? *****/
	if(n >= maxtrace) {
	    maxtrace += 1000;
	    if(realloc_global_variable() == 1) {
		fprintf(stderr, "ERROR: Memory reallocation failed.\n");
		exit(1);
	    }
	}
    }
    fclose(fp);
    return(n);
}

/*
  --------------------------------------------------------------------
*/

/********************************************************************
Read data body of SEGY file
--------------------------------------------------------------------
a           : Result amplitude
t           : time of the first sample
fp          : file pointer
bfmt        : data binary format (2:2byte int, 4:4byte int...)
data_point1 : file point (start point)
data_point2 : file point (end point)
start_time  : star time 
end_time    : end time
sampling    : sampling rate (Hz)
dt_clock    : delta t (sec)
********************************************************************/
int read_SEGY_data(float *a, float *t, FILE *fp, int bfmt,
		   int data_point1, int data_point2,
		   float start_time, float end_time,
		   float sampling, float dt_clock)
{
    /********** Local Variables **********/
    int i, i_start, i_end, count, address;
    char buf[4];
    union {
	int l;
	float a;
	char b[4];
    } u;
    
    float mkval2(char *buf, int bfmt);

    /**************************/
    /********** Main **********/
    /**************************/
    
    swap = (bfmt < 0) ? SWAP_ON : SWAP_OFF;
    
    /** start index **/
    if ( start_time + dt_clock >= 0.0 ) {
	i_start = (start_time + dt_clock)*sampling;
    } else {
	i_start = 0;
    }
    /** end index **/
    i_end = ceil((end_time + dt_clock)*sampling);
    if (i_end > data_point2)
	i_end = data_point2;
    count = i_end - i_start;
    if (count < 1) return(0);
    
    /***** read data *****/
    switch(abs(bfmt)) {
	case FMT_4BYTE_IBM:
	    address = data_point1 + i_start*4;
	    fseek(fp, address, SEEK_SET);
	    *t = (float)i_start/sampling - dt_clock;
	    for(i=0;i<count;i++){
		u.l = get_4byte_int(fp);
		ibm_to_float( &u.l );
		a[i] = u.a;
	    }
	    break;
	case FMT_4BYTE_INT:
	    address = data_point1 + i_start*4;
	    fseek(fp, address, SEEK_SET);
	    *t = (float)i_start/sampling - dt_clock;
	    fread(a, sizeof(float), count, fp);
	    /** swap **/
	    if(swap){
		for(i=0;i<count;i++){
		    u.a = a[i];
		    SWAP_4I(u.l);
		    a[i] = (float)u.l;
		}
	    } else {
		for(i=0;i<count;i++){
		    u.a = a[i];
		    a[i] = (float)u.l;
		}
	    }
	    break;
	case FMT_3BYTE_INT:
	case FMT_3BYTE_INT_SW:
	    address = data_point1 + i_start*3;
	    fseek(fp, address, SEEK_SET);
	    *t = (float)i_start/sampling - dt_clock;
	    for(i=0;i<count;i++){
		fread(buf, 1, 3, fp);
		a[i] = mkval2(buf, bfmt);
	    }
	    break;
	case FMT_2BYTE_INT:
	    address = data_point1 + i_start*2;
	    fseek(fp, address, SEEK_SET);
	    *t = (float)i_start/sampling - dt_clock;
	    for(i=0;i<count;i++){
		a[i] = (float)get_short_int(fp);
	    }
	    break;
    }
    return(count);
}
