# --------------------------------------------------------------------
# Search su file in the specified directory
#   retrun the first su file (alphabetical order).
# --------------------------------------------------------------------
# [argument]
# dir : directory name for search
# --------------------------------------------------------------------
proc su_search_datafile {dir} {
    set list [glob -nocomplain -dir $dir "*.su"]
    if { [llength $list] < 1 } {
	return "./ch1.su"
    } else {
	return [lindex $list 0]
    }
}

# --------------------------------------------------------------------
# Examine whether SU is available.
#    retunr 1 if SU mode, otherwise return 0.
# --------------------------------------------------------------------
proc check_su_mode {} {
   # global ------------------------------
   global wk

   # main ------------------------------
   if { $wk(SU,mode) } { return 1 }
   tk_messageBox -icon warning -type ok\
       -message "SU is required to use this function.  Set the environment variable CWPROOT before invoking Pasteup."
   return 0
}

# --------------------------------------------------------------------
# Check SU related environment
# retunr 1 if SU mode, otherwise return 0.
# --------------------------------------------------------------------
proc check_su_environment {} {
   global env wk
   # CWPROOT ?
   if { [lsearch [array get env] CWPROOT] < 0 || \
	    [file isdirectory $env(CWPROOT)] == 0 } {
      return 0 ; # SU filter is not capable
   }
   # SU standar command check
   update_PATH
   if { [search_external_command "suwind"] == 0 } {
      return 0 ; # standard SU command is not found
   }
   # Check advanced SU command
   if { [search_external_command "segy2su"] } {
      # have segy2su
      set wk(SU,SEGY2SU) segy2su
      catch {close $fp}
   } else {
      # don't have segy2su
      set wk(SU,SEGY2SU) segyread
   }
   return 1  ; # SU filter is capable
}

# --------------------------------------------------------------------
# Read single header value from SU file (using sugethw)
# --------------------------------------------------------------------
# [argument]
# head  : header name like "gwdep", "ns"
# --------------------------------------------------------------------
proc su_read_single_header {head} {
   # global ------------------------------
   global wk file_name
   # main ------------------------------
   if [file readable $file_name(wave)] {
      set com [open "| sugethw key=$head output=geom < $file_name(wave)"]
      gets $com val
      catch {close $com}
      return [regsub \\s $val ""]
   }
   return 0
}

# --------------------------------------------------------------------
# Get water depth at OBS from SU file
# --------------------------------------------------------------------
proc su_get_water_depth {} {
   # global ------------------------------
   global wk gbl
   # main ------------------------------
   if { $wk(SU,mode) && $gbl(data,type) == 2} {
      # SU format
      set dep [su_read_single_header gwdep]
      if { $dep > 0 } {
	 return [expr $dep / 1000.0]
      }
   } elseif { $wk(SU,mode) && $gbl(data,type) >= 5 } {
      # SEGY format
      set dep [segy_read_single_header gwdep]
      if { $dep > 0 } {
	 return [expr $dep / 1000.0]
      }
   }
   return $gbl(obs,depth)
}

# --------------------------------------------------------------------
# Read auxiliary infomation from SU file
# --------------------------------------------------------------------
proc su_read_auxiliary_info {} {
   # global ------------------------------
   global wk gbl
   # main ------------------------------
   if { $wk(SU,mode) && $gbl(data,type) == 2} {
      # read water depth at OBS
      set gbl(obs,depth) [su_get_water_depth]
   } elseif { $wk(SU,mode) && $gbl(data,type) >= 5 } {
      # read water depth at OBS
      set gbl(obs,depth) [su_get_water_depth]
   }
}

# --------------------------------------------------------------------
# Update OBS(group) position and static correction, (lon, lat, depth, stc)
# --------------------------------------------------------------------
# [argument]
# lon  : OBS longitude
# lat  : OBS latitude
# dep  : OBS depth (meter)
# stc  : static correction
# v    : water velocity
# w    : parent widget name
# --------------------------------------------------------------------
proc su_update_position {lon lat dep stc v w} {
   # global ------------------------------
   global wk gbl TMPDIR file_name pm
   # main ------------------------------
   if { [check_su_mode] == 0 } {return}
   # have the sushw ?
   if { [search_external_command susetoffset] == 0 } {
      tk_messageBox -message "\"susetoffset\" command is required to update OBS position."\
	  -type ok -icon warning -parent .
      return
   }
   # SU-data is writable?
   if { ![file writable $file_name(wave)] } {
      tk_messageBox -message "Cannot overwrite wave-data file."\
	  -type ok -icon error -parent .
      return
   }

   # update ------------------------------
   # parameters
   set tmp [format "%s/pasteup.%s.su" $TMPDIR [pid]]
   set hfile [format "%s/pasteup.%s.header" $TMPDIR [pid]]
   set bfile [format "%s/pasteup.%s.binary" $TMPDIR [pid]]
   set gx [expr int($lon * 3600.0 * 1000.0) ]
   set gy [expr int($lat * 3600.0 * 1000.0) ]
   set gwdep $dep
   set scalco -1000
   set direc [expr abs($gbl(pos,direc))]
   set orgbyte $gbl(data,byte)
   set t [expr - $stc * 1000.0]
   
   set com [format "| sushw key=gx,gy,gwdep,scalco a=%d,%d,%d,%d b=0,0,0,0\
                    | suchw key1=gstat key2=gstat a=%s\
                    | suchw key1=tstat key2=tstat a=%s\
                    | susetoffset direc=%d "\
		$gx $gy $gwdep $scalco     $t $t    $direc ]
   if { $gbl(pos,direc) < 0 } {
      set com [format "$com | suchw key1=offset key2=offset b=-1 "]
   }
   set com [make_su_command 1 $com $file_name(wave) $tmp $hfile $bfile]

   if {$gbl(data,type) > 4} {
      # SEGY: the output format must be IBM float
      if {$gbl(data,byte) == 3} { set gbl(data,byte) 4 }
      if {$gbl(data,byte) == -3} { set gbl(data,byte) -4 }
      update_data_type
   }
   set stat [exec_external_command $com "SUSETOFFSET"]
   file delete -force $hfile
   file delete -force $bfile

   # show new-waveform ------------------------------
   set backup $file_name(wave)
   set backup_vel $gbl(water_v)
   set file_name(wave) $tmp
   set gbl(water_v) $v
   su_read_auxiliary_info
   draw_pasteup
   set file_name(wave) $backup
   delete_pick_marks
   
   # confirmation ------------------------------
   set ans [tk_messageBox -type okcancel -icon question\
		-message "Update waveform file?"]
   if { $ans == "cancel" } {
      su_read_auxiliary_info
      file delete -force $tmp
      set gbl(data,byte) $orgbyte
      update_data_type
      set gbl(water_v) $backup_vel
      draw_pasteup
      destroy $w
      return
   }
   
   # overwrite waveform file
   set backup [format "%s.bak" $file_name(wave)]
   file rename -force $file_name(wave) $backup
   file rename -force $tmp $file_name(wave)
   print_message "Old wave file is backuped, as $backup"

   # change pick x-position
   remap_pick_xposition

   # change pick time : static correction
   if { $stc != 0.0 } {
      set newpm {}
      foreach d $pm {
	 scan $d "%d%s%s%s%d" sn px pt acc type
	 set pt [expr $pt - $stc]
	 lappend newpm "$sn $px $pt $acc $type"
      }
      set pm $newpm
   }
   plot_pick_marks 0

   # fin ------------------------------
   destroy $w
}

# --------------------------------------------------------------------
# Recover su-data file from the backup file
# --------------------------------------------------------------------
proc su_recover_data {} {
   # global ------------------------------
   global wk gbl file_name pm

   # main --------------------------------------------------

   # have a backup file?
   set backup [format "%s.bak" $file_name(wave)]
   if { ![file exists $backup] || ![file readable $backup] } {
      tk_messageBox -message "Cannot find/read backup file."\
	  -type ok -icon error -parent .
      return
   }
   
   # data file is writable?
   if { ![file writable $file_name(wave)] } {
      tk_messageBox -message "Cannot overwrite wave-data file."\
	  -type ok -icon error -parent .
      return
   }

   # check date
   if { [file mtime $backup] > [file mtime $file_name(wave)] } {
      tk_messageBox -icon warning -type ok\
	  -message "Backup file is newer.  Recover data file is cancelled."
      return
   }

   # recover it!
   set current_gstat [su_read_single_header gstat]
   file rename -force $backup $file_name(wave)
   set backup_gstat [su_read_single_header gstat]

   # update info
   set wk(condition) 0
   getrh makecache
   su_read_auxiliary_info

   # adjust pick
   remap_pick_xposition
   set stc [expr ($backup_gstat - $current_gstat)/1000.0]
   if { $stc != 0.0 } {
      set newpm {}
      foreach d $pm {
	 scan $d "%d%s%s%s%d" sn px pt acc type
	 set pt [expr $pt + $stc]
	 lappend newpm "$sn $px $pt $acc $type"
      }
      set pm $newpm
   }
   
   # redraw
   draw_pasteup
   plot_pick_marks 0

   print_message "$file_name(wave) is recovered from $backup.\n"
}

# --------------------------------------------------------------------
# check su filter is enabling?
# --------------------------------------------------------------------
proc enabling_su_filter_frame {} {
   # global ------------------------------
   global wk gbl custom
   # main ------------------------------
   set w $wk(w,console).frame(filter).filter
   if { $wk(SU,mode) && ( $gbl(data,type) == 2 || $gbl(data,type) >= 5 )} {
      # only enabling when SUmode with SU-data
      $w.check(sufilter) conf -fg $custom(*Foreground)
      $w.but(sufilter) conf -state normal
      widgets_update gbl(check,sufilter) $w.but(sufilter)
   } else {
      # disabling
      $w.check(sufilter) conf -fg $custom(*disabledForeground)
      $w.but(sufilter) conf -state disabled
   }
}

# --------------------------------------------------------------------
# Exec SU command for SU/SEG-Y data file
#  "header" and "binary" file will be sent to /dev/null
# --------------------------------------------------------------------
# arguments:
# mode   : 0:output is SU,  1:output is SU/SEG-Y
# com    : commands (must begin with "|")
# infile : intput data file name (SU or SEG-Y)
# outfile: output data file name (SU format)
# hfile  : header file (temporary), required if SEGY && mode==1
# bfile  : header file (temporary), required if SEGY && mode==1
# --------------------------------------------------------------------
proc make_su_command {mode com infile outfile hfile bfile} {
   # global ------------------------------
   global wk gbl

   # main ------------------------------
   if { $gbl(data,type) == 2 } {
      # data format is SU
      set newcom [format "cat %s %s > %s" $infile $com $outfile]
   } else {
      # data format is SEG-Y
      switch -- $mode {
	 0 {
	    # output file is SU
	    set newcom\
		[format "%s tape=%s endian=%d hfile=/dev/null bfile=/dev/null\
                        | segyclean %s > %s"\
			$wk(SU,SEGY2SU) $infile $gbl(su,endian)\
			$com $outfile]
	 }
	 1 {
	    # output file is SEG-Y
	    set newcom\
		[format "%s tape=%s endian=%d hfile=/dev/null bfile=/dev/null\
                        | segyclean %s \
                        | segyhdrs hfile=%s bfile=%s\
                        | %s tape=%s endian=%d hfile=%s bfile=%s" \
		     $wk(SU,SEGY2SU) $infile $gbl(su,endian)\
		     $com\
		     $hfile $bfile\
		     $wk(SU,SU2SEGY) $outfile $gbl(su,endian) $hfile $bfile]
	 }
      }
   }
   return $newcom
}
