# --------------------------------------------------------------------
# tau-p/tau-sum analysis velocity-depth window
# --------------------------------------------------------------------
proc draw_taup {} {
   # global ------------------------------
   global gbl

   # main ------------------------------
   # tau-p binding mode?
   if {$gbl(bind) != "Tau-P Mode"} { return }

   # draw water wave lines
   draw_taup_waterwave

   # plot
   plot_taup_picks
}

# --------------------------------------------------------------------
# tau-p/tau-sum analysis velocity-depth window
# --------------------------------------------------------------------
proc draw_taup_waterwave {} {
   # global ------------------------------
   global gbl wk paste

   # main ------------------------------
   if { $gbl(taup,water_wave) == 0 } {
      return
   }
   #
   set coord {}
   set water2 [expr 1/($gbl(water_v)*$gbl(water_v))]
   set depth [expr $gbl(obs,depth)]
   set maxpp [expr 1/$gbl(water_v)]
   set minpp $paste(left)
   if { $minpp < 0.0 } { set minpp 0.0 }
   for {set pp 0.0} {$pp < $maxpp} { set pp [expr $pp + 0.02] } {
      set tau [expr sqrt( $water2 - $pp*$pp) * $depth]
      if [coord inout $pp $tau] {
	 lappend coord [coord gx2cx $pp]
	 lappend coord [coord tt2cy $pp $tau]
      }
   }
   if [coord inout $maxpp 0.0] {
      lappend coord [coord gx2cx $maxpp]
      lappend coord [coord tt2cy $maxpp 0.0]
   }
   if { [llength $coord] > 3 } {
      $wk(w,canvas,c) create line $coord -width 3 -fill yellow -tags taup_water
   }
}

# --------------------------------------------------------------------
# Clear all the taup picks.
# --------------------------------------------------------------------
proc taup_clear_pick {} {
   # global --------------------
   global taup
   
   # main --------------------
   set taup(slow) {}
   set taup(tau) {}
   set taup(no) 0
   erase_all_taup_pick
   tausum_inversion
   print_message "Clear all the taup picks.\n"
}

# --------------------------------------------------------------------
# Save Tau-P pick file
# --------------------------------------------------------------------
proc taup_save_pick {} {
   # global ------------------------------
   global taup

   # main ------------------------------
   set types {
      {"tpick files"	{.tpick} }
      {"Dat files"	{.dat}	 }
      {"All files"              *}
   }
   set taup(file) [tk_getSaveFile -filetypes $types]
   if ![string compare $taup(file) ""] {
      print_message "Canceled : Save taup pick data\n"
      return
   }

   set fd [open $taup(file) w]
   puts $fd $taup(no)
   for {set i 0} {$i <= $taup(no)} {incr i} {
      set slow [lindex $taup(slow) $i]
      set tau [lindex $taup(tau) $i]
      puts $fd "$slow $tau"
   }
   close $fd
   print_message [format "%d points were written in %s.\n"\
		      $taup(no) $taup(file)]
   
}

# --------------------------------------------------------------------
# Load Tau-P pick file
# --------------------------------------------------------------------
proc taup_load_pick {} {
   # global --------------------
   global taup
   
   # main --------------------
   # file selection
   set types {
      {"tpick files"	{.tpick} }
      {"Dat files"	{.dat}	 }
      {"All files"              *}
   }
   set taup(file) [tk_getOpenFile -filetypes $types -parent . \
		       -defaultextension .tpick]
   # Open
   if { [file exist $taup(file)]} {
      exec_read_taup_pick $taup(file)
      print_message\
	  [format "%d points was read from %s.\n"\
	       $taup(no) $taup(file)]
      # plot
      erase_all_taup_pick
      plot_taup_picks
      tausum_inversion
   }
}

# --------------------------------------------------------------------
# Exec read taup-pick
# --------------------------------------------------------------------
proc exec_read_taup_pick {data_file} {
   # global --------------------
   global taup
   # main --------------------
   # reset taup data
   set taup(no) 0
   set taup(slow) {}
   set taup(tau) {}
   # open to read
   set fd [open $data_file r]
   # dummy read the number of taup pick data
   gets $fd taup(no)
   set i 0
   while {1} {
      gets $fd tmp
      if { [eof $fd] } {
	 break
      }
      if { [llength $tmp] != 2 } {
	 continue
      }
      incr i
      lappend taup(slow) [lindex $tmp 0]
      lappend taup(tau) [lindex $tmp 1]
   }
   set taup(no) $i
   close $fd
}

# --------------------------------------------------------------------
# Tau-Sum Inversion
# --------------------------------------------------------------------
proc tausum_inversion {} {
   # global --------------------
   global taup
   
   # main --------------------
   # sort taup-pick
   sort_taup_picks

   # apply tausum inversion
   apply_tausum_inversion

   # update water depth
   update_tausum_depth
}
# --------------------------------------------------------------------
# Sort taup picks in tau:monotone increase, slow:monotone decrease
# --------------------------------------------------------------------
proc sort_taup_picks {} {
   # global --------------------
   global taup

   # main --------------------
   # build a for sort
   set n [llength $taup(slow)]
   set a {}
   for {set i 0} {$i < $n} {incr i} {
      set slow [lindex $taup(slow) $i]
      set tau  [lindex $taup(tau) $i]
      lappend a "$slow $tau"
   }
   # sort in slow order (monotone decrease)
   set a [lsort -index 0 -real -decreasing $a]

   # check tau, tau must be monotone increase
   set taup(slow) {}
   set taup(tau) {}
   set prev_tau [expr [lindex [lindex $a 0] 1] - 1.0]
   foreach i $a {
      set tau  [lindex $i 1]
      if {$prev_tau > $tau} {
	 continue
      }
      lappend taup(slow) [lindex $i 0] 
      lappend taup(tau) $tau
      set prev_tau $tau
   }
   set taup(no) [llength $taup(slow)]
   erase_all_taup_pick
   plot_taup_picks
}

# --------------------------------------------------------------------
# Apply tausum inversion
# --------------------------------------------------------------------
proc apply_tausum_inversion {} {
   # global --------------------
   global taup gbl
   global vecv vecz

   # main --------------------
   if {$taup(no) < 1} { 
      vecv set {}
      vecz set {}
      return 
   }
   # sea surface
   set depth 0.0
   set vel $gbl(water_v)
   # the first pick, i.e. sea floor
   set p1 [expr 1/$vel]
   set p2 [lindex $taup(slow) 0]
   set norm [expr sqrt($p1*$p1 - $p2*$p2)]
   if { $norm <= 0.0 } {
      tk_messageBox -message "Invalid first pick.  The first pick must correspond to the sea floor." \
	  -type ok -icon error
      return
   }
   set z [expr [lindex $taup(tau) 0] / $norm]
   lappend depth $z
   lappend vel $gbl(water_v)
   lappend depth $z
   lappend vel [expr 1/$p2]

   # general picks
   set dz [lindex $depth 1]
   for {set i 1} {$i < $taup(no)} {incr i} {
      set p [lindex $taup(slow) $i]
      set d [tausum_calc_dz "$dz" $i]
      lappend dz $d
      set z [expr [lindex $depth end] + $d]
      lappend depth $z
      lappend vel [lindex $vel end]
      lappend depth $z
      lappend vel [expr 1.0/$p]
   }
   
   for {set i 0} {$i < $taup(no)} {incr i} {
      vecv set $vel
      vecz set $depth
   }
}

# --------------------------------------------------------------------
# Calculate single layer thickness
# --------------------------------------------------------------------
proc tausum_calc_dz {dz ind} {
   # global --------------------
   global taup gbl

   # main --------------------
   set p1 [expr 1/$gbl(water_v)]
   set p2 [lindex $taup(slow) $ind]
   set p2 [expr $p2 * $p2]
   set tau_prev [expr ( [lindex $taup(tau) $ind]\
			    - [lindex $dz 0] * sqrt( $p1*$p1 - $p2) ) /2. ]
   for {set i 1} {$i < [expr $ind -1]} {incr i} {
      set p1 [lindex $taup(slow) $i]
      set tau_prev [expr $tau_prev -\
			[lindex $dz $i] * sqrt( $p1*$p1 - $p2) ]
   }
   return [expr $tau_prev / sqrt( $p1*$p1 - $p2) ]
}

# --------------------------------------------------------------------
# tausum_show_positon ------------------------------------------------
#            Show the mouse cursor position on VZ-graph
# --------------------------------------------------------------------
# g   : graph widget
# l   : label widget to show position
# x, y: mouse position
# --------------------------------------------------------------------
proc tausum_show_position {g l x y} {
    # Global variables --------------------------------------------------
    global gbl pos

    # Main --------------------------------------------------
    set xz [$g invtransform $x $y]
    set v [lindex $xz 0]
    set z [lindex $xz 1]
    catch {$l conf -text [format "V=%8.3fkm/sec, Z=%8.3fkm" $v $z]}
}

# --------------------------------------------------------------------
# Save Velocity-Depth data, i.e. results of tau-sum inversion
# --------------------------------------------------------------------
proc tausum_save {} {
   # global --------------------
   global vecv vecz

   # main --------------------
   # filename, if not.
   set types {
      {"vz files"	{.vz} }
      {"All files"              *}
   }
   set vzfile [tk_getSaveFile -filetypes $types]
   if ![string compare $vzfile ""] {
      print_message "Canceled : Save vz graph\n"
      return
   }

   set n [vecv length]
   set fd [open $vzfile w]
   for {set i 0} {$i < $n} {incr i} {
      puts $fd "$vecv($i) $vecz($i)"
   }
   close $fd
   print_message [format "VZ-graph: %d points were written in %s.\n"\
		      $n $vzfile]
}
